from TransitionTSVFile cimport *
from TargetedExperiment cimport *
from DataValue cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from FeatureMap cimport *
from String cimport *
from Types cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/TargetedSpectraExtractor.h>" namespace "OpenMS":

    cdef cppclass TargetedSpectraExtractor(DefaultParamHandler):
        # wrap-inherits:
        #  DefaultParamHandler

        TargetedSpectraExtractor() nogil except +
        TargetedSpectraExtractor(TargetedSpectraExtractor) nogil except +

        void getDefaultParameters(Param&) nogil except +

        void annotateSpectra(libcpp_vector[ MSSpectrum ]&, TargetedExperiment&, libcpp_vector[ MSSpectrum ]&, FeatureMap&) nogil except +
        void annotateSpectra(libcpp_vector[ MSSpectrum ]&, TargetedExperiment&, libcpp_vector[ MSSpectrum ]&) nogil except +

        void pickSpectrum(MSSpectrum&, MSSpectrum&) nogil except +

        void scoreSpectra(libcpp_vector[ MSSpectrum ]&, libcpp_vector[ MSSpectrum ]&, FeatureMap&, libcpp_vector[ MSSpectrum ]&) nogil except +
        void scoreSpectra(libcpp_vector[ MSSpectrum ]&, libcpp_vector[ MSSpectrum ]&, libcpp_vector[ MSSpectrum ]&) nogil except +

        void selectSpectra(libcpp_vector[ MSSpectrum ]&, FeatureMap&, libcpp_vector[ MSSpectrum ]&, FeatureMap&) nogil except +
        void selectSpectra(libcpp_vector[ MSSpectrum ]&, libcpp_vector[ MSSpectrum ]&) nogil except +

        void extractSpectra(MSExperiment&, TargetedExperiment&, libcpp_vector[ MSSpectrum ]&, FeatureMap&) nogil except +
        void extractSpectra(MSExperiment&, TargetedExperiment&, libcpp_vector[ MSSpectrum ]&) nogil except +

        # void matchSpectrum(MSSpectrum& input_spectrum, TSE_Comparator& cmp, libcpp_vector[ TSE_Match ]& matches) nogil except +


cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/TargetedSpectraExtractor.h>" namespace "OpenMS::TargetedSpectraExtractor":

    cdef cppclass TSE_Match "OpenMS::TargetedSpectraExtractor::Match":

        TSE_Match() nogil except +
        TSE_Match(TSE_Match) nogil except +
        TSE_Match(MSSpectrum& spectrum, double score) nogil except +

        MSSpectrum spectrum
        double score

    # cdef cppclass TSE_Comparator "OpenMS::TargetedSpectraExtractor::Comparator":

        # TSE_Comparator() nogil except +
        # TSE_Comparator(TSE_Comparator) nogil except +

        # void generateScores(MSSpectrum& spec, libcpp_vector[libcpp_pair[Size,double]]& scores, double min_score) nogil except +
        # void init(libcpp_vector[MSSpectrum]& library, libcpp_map[String,DataValue]& options) nogil except +
        # libcpp_vector[MSSpectrum]& getLibrary() nogil except +

    # cdef cppclass TSE_BinnedSpectrumComparator "OpenMS::TargetedSpectraExtractor::BinnedSpectrumComparator" (TSE_Comparator):
        # wrap-inherits:
        #  TSE_Comparator

        # TSE_BinnedSpectrumComparator() nogil except +
        # TSE_BinnedSpectrumComparator(TSE_BinnedSpectrumComparator) nogil except +

        # void generateScores(MSSpectrum& spec, libcpp_vector[libcpp_pair[Size,double]]& scores, double min_score) nogil except +
        # void init(libcpp_vector[MSSpectrum]& library, libcpp_map[String,DataValue]& options) nogil except +



from String cimport *
from Software cimport *
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/MassAnalyzer.h>" namespace "OpenMS":

    cdef cppclass MassAnalyzer(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface

        MassAnalyzer() nogil except +
        MassAnalyzer(MassAnalyzer) nogil except + # wrap-ignore

        # returns the analyzer type
        AnalyzerType getType() nogil except +
        # sets the analyzer type
        void setType(AnalyzerType type) nogil except +

        # returns the method used for determination of the resolution
        ResolutionMethod getResolutionMethod() nogil except +
        # sets the method used for determination of the resolution
        void setResolutionMethod(ResolutionMethod resolution_method) nogil except +

        # returns the resolution type
        ResolutionType getResolutionType() nogil except +
        # sets the resolution type
        void setResolutionType(ResolutionType resolution_type) nogil except +

        # returns the direction of scanning
        ScanDirection getScanDirection() nogil except +
        # sets the direction of scanning
        void setScanDirection(ScanDirection scan_direction) nogil except +

        # returns the scan law
        ScanLaw getScanLaw() nogil except +
        # sets the scan law
        void setScanLaw(ScanLaw scan_law) nogil except +

        # returns the reflectron state (for TOF)
        ReflectronState getReflectronState() nogil except +
        # sets the reflectron state (for TOF)
        void setReflectronState(ReflectronState reflecton_state) nogil except +

        # @brief returns the resolution
        # The maximum m/z value at which two peaks can be resolved, according to one of the standard measures
        double getResolution() nogil except +
        # sets the resolution
        void setResolution(double resolution) nogil except +

        # returns the mass accuracy i.e. how much the theoretical mass may differ from the measured mass (in ppm)
        double getAccuracy() nogil except +
        # sets the accuracy  i.e. how much the theoretical mass may differ from the measured mass  (in ppm)
        void setAccuracy(double accuracy) nogil except +

        # returns the scan rate (in s)
        double getScanRate() nogil except +
        # sets the scan rate (in s)
        void setScanRate(double scan_rate) nogil except +

        # returns the scan time for a single scan (in s)
        double getScanTime() nogil except +
        # sets the scan time for a single scan (in s)
        void setScanTime(double scan_time) nogil except +

        # returns the path length for a TOF mass analyzer (in meter)
        double getTOFTotalPathLength() nogil except +
        # sets the path length for a TOF mass analyzer (in meter)
        void setTOFTotalPathLength(double TOF_total_path_length) nogil except +

        # returns the isolation width i.e. in which m/z range the precursor ion is selected for MS to the n (in m/z)
        double getIsolationWidth() nogil except +
        # sets the isolation width i.e. in which m/z range the precursor ion is selected for MS to the n (in m/z)
        void setIsolationWidth(double isolation_width) nogil except +

        # returns the final MS exponent
        Int getFinalMSExponent() nogil except +
        # sets the final MS exponent
        void setFinalMSExponent(Int final_MS_exponent) nogil except +

        # returns the strength of the magnetic field (in T)
        double getMagneticFieldStrength() nogil except +
        # sets the strength of the magnetic field (in T)
        void setMagneticFieldStrength(double magnetic_field_strength) nogil except +

        #
        #   @brief returns the position of this part in the whole Instrument.
        #
        #   Order can be ignored, as long the instrument has this default setup:
        #   - one ion source
        #   - one or many mass analyzers
        #   - one ion detector
        #
        #   For more complex instruments the order should be defined.
        Int getOrder() nogil except +
        # sets the order
        void setOrder(Int order) nogil except +

cdef extern from "<OpenMS/METADATA/MassAnalyzer.h>" namespace "OpenMS::MassAnalyzer":

    # analyzer type
    cdef enum AnalyzerType:
      # wrap-attach:
      #     MassAnalyzer
      ANALYZERNULL,                         #< Unknown
      QUADRUPOLE,                           #< Quadrupole
      PAULIONTRAP,                          #< Quadrupole ion trap / Paul ion trap
      RADIALEJECTIONLINEARIONTRAP,          #< Radial ejection linear ion trap
      AXIALEJECTIONLINEARIONTRAP,           #< Axial ejection linear ion trap
      TOF,                                  #< Time-of-flight
      SECTOR,                               #< Magnetic sector
      FOURIERTRANSFORM,                     #< Fourier transform ion cyclotron resonance mass spectrometer
      IONSTORAGE,                           #< Ion storage
      ESA,                                  #< Electrostatic energy analyzer
      IT,                                   #< Ion trap
      SWIFT,                                #< Stored waveform inverse fourier transform
      CYCLOTRON,                            #< Cyclotron
      ORBITRAP,                             #< Orbitrap
      LIT,                                  #< Linear ion trap
      SIZE_OF_ANALYZERTYPE


    # Which of the available standard measures is used to define whether two peaks are separate
    cdef enum ResolutionMethod:
      # wrap-attach:
      #     MassAnalyzer
      RESMETHNULL,                  #< Unknown
      FWHM,                         #< Full width at half max
      TENPERCENTVALLEY,             #< Ten percent valley
      BASELINE,                     #< Baseline
      SIZE_OF_RESOLUTIONMETHOD

    # Resolution type
    cdef enum ResolutionType:
      # wrap-attach:
      #     MassAnalyzer
      RESTYPENULL,              #< Unknown
      CONSTANT,                 #< Constant
      PROPORTIONAL,             #< Proportional
      SIZE_OF_RESOLUTIONTYPE

    # direction of scanning
    cdef enum ScanDirection:
      # wrap-attach:
      #     MassAnalyzer
      SCANDIRNULL,              #< Unknown
      UP,                       #< Up
      DOWN,                     #< Down
      SIZE_OF_SCANDIRECTION

    #Scan law
    cdef enum ScanLaw:
      # wrap-attach:
      #     MassAnalyzer
      SCANLAWNULL,              #< Unknown
      EXPONENTIAL,              #< Unknown
      LINEAR,                   #< Linear
      QUADRATIC,                #< Quadratic
      SIZE_OF_SCANLAW

    #Reflectron state
    cdef enum ReflectronState:
      # wrap-attach:
      #     MassAnalyzer
      REFLSTATENULL,            #< Unknown
      ON,                       #< On
      OFF,                      #< Off
      NONE,                     #< None
      SIZE_OF_REFLECTRONSTATE




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from MSSpectrum cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/CalibrationData.h>" namespace "OpenMS":

    cdef cppclass CalibrationData:

        CalibrationData()  nogil except +
        CalibrationData(CalibrationData &) nogil except +
        double getMZ(Size) nogil except +
        double getRT(Size) nogil except +
        double getIntensity(Size) nogil except +
        Size size() nogil except +
        bool empty() nogil except +
        void clear() nogil except +
        void setUsePPM(bool) nogil except +
        bool usePPM() nogil except +
        void insertCalibrationPoint(double rt, double mz_obs, float intensity, double mz_ref, double weight, int group) nogil except +
        Size getNrOfGroups() nogil except +
        double getError(Size) nogil except +
        double getRefMZ(Size) nogil except +
        double getWeight(Size) nogil except +
        int getGroup(Size i) nogil except +
        CalibrationData median(double, double) nogil except +
        void sortByRT() nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/CalibrationData.h>" namespace "OpenMS::CalibrationData":

        # static members
        StringList getMetaValues() nogil except + # wrap-attach:CalibrationData






    def getAAFrequencies(self, dict mmap):
        cdef Map[_String, size_t] c_mmap 
        for k,v in mmap.iteritems():
            c_mmap[ _String(<char *>k) ] = v # add <size_t> ?
        self.inst.get().getAAFrequencies(c_mmap)

    def __iter__(self):
        cdef int n = self.inst.get().size()
        cdef int i = 0

        cdef Residue py_result 
        while i < n:

            py_result = Residue.__new__(Residue)
            py_result.inst = shared_ptr[_Residue](new _Residue( deref(self.inst.get())[i] ))

            yield py_result

            i += 1




from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from ChargePair cimport *
from FeatureMap cimport *
from Feature cimport *

# ctypedef std::vector<ChargePair> PairsType;
cdef extern from "<OpenMS/ANALYSIS/DECHARGING/ILPDCWrapper.h>" namespace "OpenMS":
    
    cdef cppclass ILPDCWrapper "OpenMS::ILPDCWrapper":
        ILPDCWrapper() nogil except +
        ILPDCWrapper(ILPDCWrapper) nogil except + #wrap-ignore
        double compute(FeatureMap fm, libcpp_vector[ChargePair] & pairs, Size verbose_level) nogil except +




from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from SavitzkyGolayFilter cimport *
from PeakPickerHiRes cimport *
from MzTab cimport *
from MzMLFile cimport *
from FeatureMapping cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/FIAMSDataProcessor.h>" namespace "OpenMS":

    cdef cppclass FIAMSDataProcessor(DefaultParamHandler) :
        #
        # wrap-doc:
        #     ADD PYTHON DOCUMENTATION HERE
        #
        # wrap-inherits:
        #  DefaultParamHandler
        FIAMSDataProcessor() nogil except +
        FIAMSDataProcessor(FIAMSDataProcessor) nogil except +
        bool run(MSExperiment & experiment, float & n_seconds, MzTab & output, bool load_cached_spectrum) nogil except +
        # void cutForTime(MSExperiment & experiment, float & n_seconds, libcpp_vector[ MSSpectrum ] & output) nogil except +
        # NAMESPACE # MSSpectrum mergeAlongTime(libcpp_vector[ OpenMS::MSSpectrum ] & input_) nogil except +
        MSSpectrum extractPeaks(MSSpectrum & input_) nogil except +
        FeatureMap convertToFeatureMap(MSSpectrum & input_) nogil except +
        MSSpectrum trackNoise(MSSpectrum & input_) nogil except +
        # NAMESPACE # void runAccurateMassSearch(FeatureMap & input_, OpenMS::MzTab & output) nogil except +
        # libcpp_vector[ float ] getMZs() nogil except +
        # libcpp_vector[ float ] getBinSizes() nogil except +




from Types cimport *
from String cimport *
from Residue cimport *

cdef extern from "<OpenMS/CHEMISTRY/ResidueDB.h>" namespace "OpenMS":
    
    cdef cppclass ResidueDB "OpenMS::ResidueDB":
        # wrap-manual-memory:
        #   cdef AutowrapPtrHolder[_ResidueDB] inst

        ResidueDB(ResidueDB) nogil except + #wrap-ignore

        Size getNumberOfResidues() nogil except +
        Size getNumberOfModifiedResidues() nogil except +
        const Residue * getResidue(const String & name) nogil except +
        const Residue * getModifiedResidue(const String & name) nogil except +
        const Residue * getModifiedResidue(Residue * residue, const String & name) nogil except +
        libcpp_set[ const Residue * ] getResidues(const String & residue_set) nogil except +
        libcpp_set[ String ] getResidueSets() nogil except +
        bool hasResidue(const String & name) nogil except +
        # bool hasResidue(Residue * residue) nogil except + # does not really work as the ptr is different

# COMMENT: wrap static methods
cdef extern from "<OpenMS/CHEMISTRY/ResidueDB.h>" namespace "OpenMS::ResidueDB":
    
    ResidueDB* getInstance() nogil except + # wrap-ignore




from Types cimport *
from DigestionEnzyme cimport *
from StringView cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/CHEMISTRY/EnzymaticDigestion.h>" namespace "OpenMS":

    cdef cppclass EnzymaticDigestion "OpenMS::EnzymaticDigestion":
        EnzymaticDigestion(EnzymaticDigestion) nogil except +
        EnzymaticDigestion() nogil except +

        # const String NamesOfSpecificity[SIZE_OF_SPECIFICITY]
        # const String UnspecificCleavage

        Size getMissedCleavages() nogil except +
        void setMissedCleavages(Size missed_cleavages) nogil except +
        String getEnzymeName() nogil except +
        void setEnzyme(DigestionEnzyme* enzyme) nogil except +
        Specificity getSpecificity() nogil except +
        void setSpecificity(Specificity spec) nogil except +
        Specificity getSpecificityByName(const String& name) nogil except +

        Size digestUnmodified(StringView sequence,
                              libcpp_vector[ StringView ]& output,
                              Size min_length,
                              Size max_length) nogil except +

        bool isValidProduct(const String& sequence,
                            int pos, int length,
                            bool ignore_missed_cleavages) nogil except +

        # bool filterByMissedCleavages(const String& sequence,
        #                              std::function<bool(Int)> filter) nogil except +


cdef extern from "<OpenMS/CHEMISTRY/EnzymaticDigestion.h>" namespace "OpenMS::EnzymaticDigestion":
    cdef enum Specificity "OpenMS::EnzymaticDigestion::Specificity":
        #wrap-attach:
        #    EnzymaticDigestion
        CROSS
        MONO
        LOOP
        NUMBER_OF_CROSS_LINK_TYPES



from Types cimport *
from MSChromatogram cimport *
from MSSpectrum cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/MSDataStoringConsumer.h>" namespace "OpenMS":
    
    cdef cppclass MSDataStoringConsumer :

        MSDataStoringConsumer() nogil except +
        MSDataStoringConsumer(MSDataStoringConsumer) nogil except + #wrap-ignore

        void setExperimentalSettings(ExperimentalSettings & exp) nogil except +
        void setExpectedSize(Size expectedSpectra, Size expectedChromatograms) nogil except +

        void consumeSpectrum(MSSpectrum & s) nogil except +
        void consumeChromatogram(MSChromatogram & ) nogil except +

        MSExperiment getData() nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from RichPeak2D cimport *
from UniqueIdInterface cimport *
from BaseFeature cimport *
from PeptideIdentification cimport *

cdef extern from "<OpenMS/KERNEL/BaseFeature.h>" namespace "OpenMS":

    cdef cppclass BaseFeature(UniqueIdInterface, RichPeak2D):
        # wrap-inherits:
        #    UniqueIdInterface
        #    RichPeak2D

        BaseFeature()  nogil except +
        BaseFeature(BaseFeature &) nogil except +

        float getQuality()  nogil except +
        void setQuality(float q) nogil except +

        float getWidth() nogil except +
        void setWidth(float q) nogil except +

        Int getCharge() nogil except +
        void setCharge(Int q) nogil except +
        AnnotationState getAnnotationState() nogil except +

        # returns a mutable reference to the PeptideIdentification vector
        libcpp_vector[PeptideIdentification] getPeptideIdentifications() nogil except +
        # sets the PeptideIdentification vector
        void setPeptideIdentifications(libcpp_vector[PeptideIdentification] & peptides) nogil except +

        bool operator==(BaseFeature) nogil except +
        bool operator!=(BaseFeature) nogil except +

cdef extern from "<OpenMS/KERNEL/BaseFeature.h>" namespace "OpenMS::BaseFeature":
    
    cdef enum AnnotationState "OpenMS::BaseFeature::AnnotationState":
        FEATURE_ID_NONE
        FEATURE_ID_SINGLE
        FEATURE_ID_MULTIPLE_SAME
        FEATURE_ID_MULTIPLE_DIVERGENT
        SIZE_OF_ANNOTATIONSTATE



from libcpp cimport bool
from Types cimport *
from String cimport *

cdef extern from "<OpenMS/SYSTEM/JavaInfo.h>" namespace "OpenMS":

    cdef cppclass JavaInfo:

        JavaInfo() nogil except +
        JavaInfo(JavaInfo) nogil except + # wrap-ignore

        bool canRun(String java_executable) nogil except +




from MSExperiment  cimport *
from MSSpectrum  cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from String cimport *
from ProgressLogger cimport *
from streampos cimport *

cdef extern from "<OpenMS/FORMAT/HANDLERS/CachedMzMLHandler.h>" namespace "OpenMS::Internal":

    # Do not use this class directly, rather use SpectrumAccessOpenMSCached

    cdef cppclass CachedMzMLHandler(ProgressLogger):
        # wrap-inherits:
        #   ProgressLogger

        CachedMzMLHandler() nogil except +
        CachedMzMLHandler(CachedMzMLHandler) nogil except +

        void writeMemdump(MSExperiment exp, String out) nogil except +
        void writeMetadata(MSExperiment exp, String out_meta) nogil except +

        void readMemdump(MSExperiment exp, String filename) nogil except +

        # void readSingleSpectrum(MSSpectrum & spectrum, String & filename, Size & idx) nogil except +
        libcpp_vector[ streampos ]  getSpectraIndex() nogil except +
        libcpp_vector[ streampos ]  getChromatogramIndex() nogil except +
        void createMemdumpIndex(String filename) nogil except +
        # NAMESPACE # void readSingleSpectrum(MSSpectrum & spectrum, std::ifstream & ifs, Size & idx)
        # NAMESPACE # void readSpectrumFast(OpenSwath::BinaryDataArrayPtr data1, OpenSwath::BinaryDataArrayPtr data2, std::ifstream & ifs, int ms_level, double rt)
        # NAMESPACE # void readChromatogramFast(OpenSwath::BinaryDataArrayPtr data1, OpenSwath::BinaryDataArrayPtr data2, std::ifstream & ifs)




from libcpp.vector cimport vector as libcpp_vector
from String cimport *
from CVTerm cimport *
from Residue cimport *
from Map cimport *
from DataValue cimport *
from CVTermList cimport *

cdef extern from "<OpenMS/ANALYSIS/TARGETED/TargetedExperimentHelper.h>" namespace "OpenMS::TargetedExperimentHelper::RetentionTime::RTUnit":

    ctypedef enum RTUnit "OpenMS::TargetedExperimentHelper::RetentionTime::RTUnit":
            # wrap-attach:
            #    RetentionTime
            SECOND,        # RT stored in seconds
            MINUTE,        # RT stored in minutes
            UNKNOWN,       # no stored annotation
            SIZE_OF_RTUNIT

cdef extern from "<OpenMS/ANALYSIS/TARGETED/TargetedExperimentHelper.h>" namespace "OpenMS::TargetedExperimentHelper::RetentionTime::RTType":

    ctypedef enum RTType "OpenMS::TargetedExperimentHelper::RetentionTime::RTType":
            # wrap-attach:
            #    RetentionTime
            LOCAL,            # undefined local chromatography
            NORMALIZED,       # standardized reference chromatography
            PREDICTED,        # predicted by referenced software
            HPINS,            # H-PINS "The de facto standard providing the retention times"
            IRT,              # iRT retention time standard
            UNKNOWN,          # no stored annotation
            SIZE_OF_RTTYPE

cdef extern from "<OpenMS/ANALYSIS/TARGETED/TargetedExperimentHelper.h>" namespace "OpenMS::TargetedExperimentHelper":

    cdef cppclass Configuration(CVTermList):
        # wrap-inherits:
        #    CVTermList

        Configuration(Configuration) nogil except + #wrap-ignore
        String contact_ref
        String instrument_ref
        libcpp_vector[ CVTermList ] validations


    cdef cppclass CV:
        CV(CV) nogil except +
        CV(String new_id, String new_fullname, String new_version, String new_URI)  nogil except +

        String id
        String fullname
        String version
        String URI

    cdef cppclass Protein(CVTermList):
        # wrap-inherits:
        #    CVTermList

        Protein() nogil except +
        Protein(Protein) nogil except +

        String id
        String sequence


    cdef cppclass RetentionTime(CVTermList):
        # wrap-inherits:
        #    CVTermList

        RetentionTime() nogil except +
        RetentionTime(RetentionTime) nogil except +

        String software_ref
        RTUnit retention_time_unit
        RTType retention_time_type

        bool isRTset() nogil except + 
        void setRT(double rt) nogil except +
        double getRT() nogil except +

    cdef cppclass Compound(CVTermList):
        # wrap-inherits:
        #    CVTermList

        Compound() nogil except +
        Compound(Compound) nogil except +
        bool operator==(Compound & rhs) nogil except +

        String id
        String molecular_formula
        String smiles_string
        double theoretical_mass
        libcpp_vector[ RetentionTime ] rts

        void setChargeState(int charge) nogil except +
        int getChargeState() nogil except +
        bool hasCharge() nogil except +
        double getRetentionTime() nogil except +
        bool hasRetentionTime() nogil except +
        RTType getRetentionTimeType() nogil except +
        RTUnit getRetentionTimeUnit() nogil except +

    cdef cppclass Peptide(CVTermList):
        # wrap-inherits:
        #    CVTermList

        Peptide() nogil except +
        Peptide(Peptide) nogil except +

        # members
        libcpp_vector[RetentionTime] rts
        String id
        libcpp_vector[String] protein_refs
        CVTermList evidence
        String sequence
        libcpp_vector[TargetedExperiment_Modification] mods

        void setPeptideGroupLabel(String label) nogil except +
        String getPeptideGroupLabel() nogil except +

        void setChargeState(int charge) nogil except +
        int getChargeState() nogil except +
        bool hasCharge() nogil except +
        double getRetentionTime() nogil except +
        bool hasRetentionTime() nogil except +
        RTType getRetentionTimeType() nogil except +
        RTUnit getRetentionTimeUnit() nogil except +

    cdef cppclass Contact(CVTermList):
        # wrap-inherits:
        #    CVTermList

        Contact() nogil except +
        Contact(Contact) nogil except + #wrap-ignore
        String id
        bool operator==(Contact & rhs) nogil except +

    cdef cppclass Publication(CVTermList):
        # wrap-inherits:
        #    CVTermList

        Publication() nogil except +
        Publication(Publication) nogil except + #wrap-ignore
        String id
        bool operator==(Publication & rhs) nogil except +


    cdef cppclass TargetedExperiment_Instrument "OpenMS::TargetedExperimentHelper::Instrument":
        TargetedExperiment_Instrument() nogil except +
        TargetedExperiment_Instrument(TargetedExperiment_Instrument) nogil except + #wrap-ignore
        String id
        bool operator==(TargetedExperiment_Instrument & rhs) nogil except +

        # CVTermList:
        void setCVTerms(libcpp_vector[CVTerm] & terms)  nogil except +
        void replaceCVTerm(CVTerm & term)               nogil except +
        void replaceCVTerms(libcpp_vector[CVTerm] cv_terms, String accession) nogil except +
        void replaceCVTerms(Map[String, libcpp_vector[CVTerm] ] cv_term_map) nogil except +
        void consumeCVTerms(Map[String, libcpp_vector[CVTerm] ] cv_term_map) nogil except +
        Map[String, libcpp_vector[CVTerm] ] getCVTerms() nogil except +
        void addCVTerm(CVTerm & term)                   nogil except +
        bool hasCVTerm(String accession)  nogil except +
        bool empty()                      nogil except +

        # MetaInfoInterface:
        # cython has a problem with inheritance of overloaded methods,
        # so we do not declare them here, but separately in each derived
        # class which we want to be wrapped:
        void getKeys(libcpp_vector[String] & keys) nogil except +
        void getKeys(libcpp_vector[unsigned int] & keys) nogil except + # wrap-as:getKeysAsIntegers
        DataValue getMetaValue(unsigned int) nogil except +
        DataValue getMetaValue(String) nogil except +
        void setMetaValue(unsigned int, DataValue) nogil except +
        void setMetaValue(String, DataValue) nogil except +
        bool metaValueExists(String) nogil except +
        bool metaValueExists(unsigned int) nogil except +
        void removeMetaValue(String) nogil except +
        void removeMetaValue(unsigned int) nogil except +

    cdef cppclass Prediction(CVTermList):
        # wrap-inherits:
        #    CVTermList

        Prediction() nogil except +
        Prediction(Prediction) nogil except + #wrap-ignore
        bool operator==(Prediction & rhs) nogil except +

        String software_ref
        String contact_ref


    cdef cppclass TargetedExperiment_Interpretation "OpenMS::TargetedExperimentHelper::Interpretation":
        TargetedExperiment_Interpretation() nogil except +
        TargetedExperiment_Interpretation(TargetedExperiment_Interpretation) nogil except + #wrap-ignore

        unsigned char ordinal
        unsigned char rank
        ResidueType iontype

        # CVTermList:
        void setCVTerms(libcpp_vector[CVTerm] & terms)  nogil except +
        void replaceCVTerm(CVTerm & term)               nogil except +
        void replaceCVTerms(libcpp_vector[CVTerm] cv_terms, String accession) nogil except +
        void replaceCVTerms(Map[String, libcpp_vector[CVTerm] ] cv_term_map) nogil except +
        void consumeCVTerms(Map[String, libcpp_vector[CVTerm] ] cv_term_map) nogil except +
        Map[String, libcpp_vector[CVTerm] ] getCVTerms() nogil except +
        void addCVTerm(CVTerm & term)                   nogil except +
        bool hasCVTerm(String accession)  nogil except +
        bool empty()                      nogil except +

        # MetaInfoInterface:
        # cython has a problem with inheritance of overloaded methods,
        # so we do not declare them here, but separately in each derived
        # class which we want to be wrapped:
        void getKeys(libcpp_vector[String] & keys) nogil except +
        void getKeys(libcpp_vector[unsigned int] & keys) nogil except + # wrap-as:getKeysAsIntegers
        DataValue getMetaValue(unsigned int) nogil except +
        DataValue getMetaValue(String) nogil except +
        void setMetaValue(unsigned int, DataValue) nogil except +
        void setMetaValue(String, DataValue) nogil except +
        bool metaValueExists(String) nogil except +
        bool metaValueExists(unsigned int) nogil except +
        void removeMetaValue(String) nogil except +
        void removeMetaValue(unsigned int) nogil except +

    cdef cppclass TraMLProduct(CVTermList):
        # wrap-inherits:
        #    CVTermList

        TraMLProduct() nogil except +
        TraMLProduct(TraMLProduct) nogil except + #wrap-ignore
        bool operator==(TraMLProduct & rhs) nogil except +

        void setMZ(double mz) nogil except +
        double getMZ() nogil except +

        void setChargeState(int charge) nogil except +
        int getChargeState() nogil except +
        bool hasCharge() nogil except +
        libcpp_vector[ Configuration ]  getConfigurationList() nogil except +
        void addConfiguration(Configuration configuration) nogil except +
        libcpp_vector[ TargetedExperiment_Interpretation ] getInterpretationList() nogil except +
        void addInterpretation(TargetedExperiment_Interpretation interpretation) nogil except +
        void resetInterpretations() nogil except +


# no support for nested classes yet in Cython
cdef extern from "<OpenMS/ANALYSIS/TARGETED/TargetedExperimentHelper.h>" namespace "OpenMS::TargetedExperimentHelper::Peptide":

    cdef cppclass TargetedExperiment_Modification "OpenMS::TargetedExperimentHelper::Peptide::Modification":
        TargetedExperiment_Modification() nogil except +
        TargetedExperiment_Modification(TargetedExperiment_Modification) nogil except +

        # members
        double avg_mass_delta
        double mono_mass_delta
        int location
        int unimod_id




from Types cimport *
from String cimport *
from InterfaceDataStructures cimport *

cdef extern from "<OpenMS/FORMAT/HANDLERS/MzMLSpectrumDecoder.h>" namespace "OpenMS":
    
    cdef cppclass MzMLSpectrumDecoder "OpenMS::MzMLSpectrumDecoder":
        MzMLSpectrumDecoder() nogil except +
        MzMLSpectrumDecoder(MzMLSpectrumDecoder) nogil except +
        void domParseChromatogram(String in_, shared_ptr[Chromatogram] & cptr) nogil except +
        void domParseSpectrum(String in_, shared_ptr[Spectrum] & cptr) nogil except +
        void setSkipXMLChecks(bool only) nogil except +




from libcpp cimport bool
from ProgressLogger cimport *
from UniqueIdGenerator cimport *
from String cimport *

cdef extern from "<OpenMS/FORMAT/QcMLFile.h>" namespace "OpenMS::QcMLFile":
    
    cdef cppclass Attachment "OpenMS::QcMLFile::Attachment":
        Attachment() nogil except +
        Attachment(Attachment) nogil except +
        String name
        String id
        String value
        String cvRef
        String cvAcc
        String unitRef
        String unitAcc
        String binary
        String qualityRef
        libcpp_vector[ String ] colTypes
        libcpp_vector[ libcpp_vector[ String ] ] tableRows # wrap-ignore
        bool operator==(Attachment &rhs) nogil except +
        bool operator<(Attachment &rhs) nogil except +
        bool operator>(Attachment &rhs) nogil except +
        String toXMLString(UInt indentation_level) nogil except +
        String toCSVString(String separator) nogil except +




import unittest
import os

import pyopenms

class TestMorphologicalFilter(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))
        self.filename = os.path.join(dirname, "test2.mzML").encode()
        self.exp = pyopenms.MSExperiment()
        pyopenms.MzMLFile().load(self.filename, self.exp)

    def test_init(self):
        thisfilter = pyopenms.MorphologicalFilter();

    def test_run(self):
        thisfilter = pyopenms.MorphologicalFilter();
        old_firstspec = self.exp[0]
        # needs different parameters to have any effect ...
        params = pyopenms.MorphologicalFilter().getDefaults();
        params.setValue(b"struc_elem_length", 0.05, b'')
        thisfilter.setParameters(params);
        thisfilter.filterExperiment(self.exp)

        self.assertNotEqual(self.exp.size(), 0)
        self.assertNotEqual(old_firstspec, self.exp[0])

        # MZ should not change, Intensity should
        self.assertEqual(old_firstspec[10].getMZ(), self.exp[0][10].getMZ())
        self.assertNotEqual(old_firstspec[10].getIntensity(), self.exp[0][10].getIntensity())

if __name__ == '__main__':
    unittest.main()



from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from libcpp.vector cimport vector as libcpp_vector
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/FIAMSScheduler.h>" namespace "OpenMS":

    cdef cppclass FIAMSScheduler "OpenMS::FIAMSScheduler":
        #
        # wrap-doc:
        #     ADD PYTHON DOCUMENTATION HERE
        #
        FIAMSScheduler() nogil except +
        FIAMSScheduler(FIAMSScheduler) nogil except +
        FIAMSScheduler(String filename, String base_dir, bool load_cached_) nogil except +
        void run() nogil except +
        # libcpp_vector[ libcpp_map[ String, String ] ] getSamples() nogil except +
        String getBaseDir() nogil except +




import os,sys

import pyopenms

"""

python pyTOPP/OpenSwathChromatogramExtractor.py --in ../source/TEST/TOPP/OpenSwathChromatogramExtractor_input.mzML\
        --tr ../source/TEST/TOPP/OpenSwathChromatogramExtractor_input.TraML --out OpenSwathChromatogramExtractor.tmp.out \
        && diff OpenSwathChromatogramExtractor.tmp.out ../source/TEST/TOPP/OpenSwathChromatogramExtractor_output.mzML

"""

def main(options):

    # load TraML file
    targeted = pyopenms.TargetedExperiment();
    pyopenms.TraMLFile().load(options.traml_in, targeted);

    # Create empty files as input and finally as output
    empty_swath = pyopenms.MSExperiment()
    trafo = pyopenms.TransformationDescription()
    output = pyopenms.MSExperiment();

    # load input
    for infile in options.infiles:
        exp = pyopenms.MSExperiment()
        pyopenms.FileHandler().loadExperiment(infile, exp)

        transition_exp_used = pyopenms.TargetedExperiment();

        do_continue = True
        if options.is_swath:
            do_continue = pyopenms.OpenSwathHelper().checkSwathMapAndSelectTransitions(exp, targeted, transition_exp_used, options.min_upper_edge_dist)
        else:
            transition_exp_used = targeted

        if do_continue:
            # set up extractor and run
            tmp_out = pyopenms.MSExperiment();
            extractor = pyopenms.ChromatogramExtractor()
            extractor.extractChromatograms(exp, tmp_out, targeted, options.extraction_window, options.ppm, trafo, options.rt_extraction_window, options.extraction_function)
            # add all chromatograms to the output
            for chrom in tmp_out.getChromatograms():
                output.addChromatogram(chrom)

    dp = pyopenms.DataProcessing()
    pa = pyopenms.ProcessingAction().SMOOTHING
    dp.setProcessingActions(set([pa]))

    chromatograms = output.getChromatograms();
    for chrom in chromatograms:
        this_dp = chrom.getDataProcessing()
        this_dp.append(dp)
        chrom.setDataProcessing(this_dp)

    output.setChromatograms(chromatograms);

    pyopenms.MzMLFile().store(options.outfile, output);

def handle_args():
    import argparse

    usage = ""
    usage += "\nExtract chromatograms (XIC) from a MS2 map file."

    parser = argparse.ArgumentParser(description = usage )
    parser.add_argument('--in', dest="infiles", nargs = '+', help = 'An input file containing spectra')
    parser.add_argument("--tr", dest="traml_in", help="TraML input file containt the transitions")
    parser.add_argument("--out", dest="outfile", help="Output chrom.mzML file with chromatograms")
    parser.add_argument("--extraction_window", dest="extraction_window", default=0.05, help="Extraction window in Th", metavar='0.05', type=float)
    parser.add_argument("--min_upper_edge_dist", dest="min_upper_edge_dist", default=0.0, help="Minimal distance to the edge to still consider a precursor, in Thomson (for Swath)", metavar='0.0', type=float)
    parser.add_argument('--ppm', action='store_true', default=False, help="use ppm instead of Th")
    parser.add_argument('--is_swath', action='store_true', default=False, help="The input file is a SWATH file")
    parser.add_argument("--rt_extraction_window", dest="rt_extraction_window", default=-1, help="Extraction window in RT", metavar='-1', type=float)
    parser.add_argument("--extraction_function", dest="extraction_function", default="tophat", help="Extraction function (tophat or bartlett)", metavar="tophat")

    args = parser.parse_args(sys.argv[1:])
    return args

if __name__ == '__main__':
    options = handle_args()
    main(options)




import unittest
import os

import pyopenms

def general_setup(self):
    self.eightplex = pyopenms.ItraqEightPlexQuantitationMethod()
    self.fourplex = pyopenms.ItraqFourPlexQuantitationMethod()
    self.tmt = pyopenms.TMTSixPlexQuantitationMethod()

class TestIsobaricQuantitationMethod(unittest.TestCase):

    def setUp(self):
        pass

    def test_init_fxn(self):
        eightplex = pyopenms.ItraqEightPlexQuantitationMethod()
        fourplex = pyopenms.ItraqFourPlexQuantitationMethod()
        tmt = pyopenms.TMTSixPlexQuantitationMethod()

        for inst in [eightplex, fourplex, tmt]:
            assert inst.getName() is not None
            assert inst.getChannelInformation() is not None
            assert inst.getNumberOfChannels() is not None
            assert inst.getIsotopeCorrectionMatrix() is not None
            assert inst.getReferenceChannel() is not None

        self.assertEqual(tmt.getIsotopeCorrectionMatrix().rows(), 6)
        self.assertEqual(tmt.getIsotopeCorrectionMatrix().cols(), 6)

        self.assertEqual(fourplex.getIsotopeCorrectionMatrix().rows(), 4)
        self.assertEqual(fourplex.getIsotopeCorrectionMatrix().cols(), 4)

        self.assertEqual(eightplex.getIsotopeCorrectionMatrix().rows(), 8)
        self.assertEqual(eightplex.getIsotopeCorrectionMatrix().cols(), 8)

class TestIsobaricChannelExtractor(unittest.TestCase):

    def setUp(self):
        general_setup(self)

    def testInit(self):
        assert pyopenms.IsobaricChannelExtractor(self.eightplex) is not None
        assert pyopenms.IsobaricChannelExtractor(self.fourplex) is not None
        assert pyopenms.IsobaricChannelExtractor(self.tmt) is not None

    def testFunction(self):

        for method in [self.eightplex, self.fourplex, self.tmt]:
            inst = pyopenms.IsobaricChannelExtractor(method)
            map1 = pyopenms.ConsensusMap()
            exp = pyopenms.MSExperiment()
            assert inst.extractChannels is not None
            # Will not work with empty map
            # inst.extractChannels(exp, map1)

class TestIsobaricNormalizer(unittest.TestCase):

    def setUp(self):
        general_setup(self)

    def testInit(self):
        assert pyopenms.IsobaricNormalizer(self.eightplex) is not None
        assert pyopenms.IsobaricNormalizer(self.fourplex) is not None
        assert pyopenms.IsobaricNormalizer(self.tmt) is not None

    def testFunction(self):

        for method in [self.eightplex, self.fourplex, self.tmt]:
            inst = pyopenms.IsobaricNormalizer(method)
            map1 = pyopenms.ConsensusMap()
            assert inst.normalize is not None
            inst.normalize(map1)

#class TestIsobaricIsotopeCorrector(unittest.TestCase):

#    def setUp(self):
#        general_setup(self)

    #method not testable since it contains inherited class member in its signature (is there a way to test it?)
    #def testFunction(self):

        #for method in [self.eightplex, self.fourplex, self.tmt]:
        #    assert correctIsotopicImpurities(map1, map2, method)


if __name__ == '__main__':
    unittest.main()







    def __getitem__(self,  in_0 ):
        assert isinstance(in_0, (int, long)), 'arg key wrong type'
    
        cdef long _idx = (<int>in_0)
        if _idx >= self.inst.get().size():
            raise IndexError("invalid index %d" % _idx)

        cdef float _r = deref(self.inst.get())[(<int>in_0)]
        py_result = <float>_r
        return py_result

    def __setitem__(self, key, value):
        assert isinstance(key, (int, long)), 'arg key wrong type'
        assert isinstance(value, (int, long, float)), 'arg value wrong type'
        cdef long _idx = (<int>key)
        if _idx >= self.inst.get().size():
            raise IndexError("invalid index %d" % _idx)

        cdef float _v = (<float>value)
        deref( self.inst.get() )[(<int>key)] = _v

    def get_data(self):
        """
        Gets the raw data for the float data array

        Example usage: 

          fd = pyopenms.FloatDataArray()
          data = fd.get_data()

        """
        cdef _FloatDataArray * fda_ = self.inst.get()
        cdef unsigned int n = fda_.size()
         
        # Obtain a raw ptr to the beginning of the C++ array
        cdef libcpp_vector[float] * vec_ptr = <libcpp_vector[float]*> fda_
        cdef float * raw_ptr =  address(deref(vec_ptr)[0]) 

        # We use a memory view to get the data from the raw data
        # See https://cython.readthedocs.io/en/latest/src/userguide/memoryviews.html 
        # See https://stackoverflow.com/questions/43021574/cast-c-array-into-numpy-array-cython-typed-memoryview-in-cython-code
        cdef float[:] fda_view = <float[:n]>raw_ptr # cast to memoryview, refer to the underlying buffer without copy
        xarr = np.asarray(fda_view) # numpy array refer to the underlying buffer without copy
        return xarr

        ## # Alternatively, we could also use the assignment method:
        ## cdef _FloatDataArray * fda_ = self.inst.get()
        ## cdef libcpp_vector[float].iterator it = fda_.begin()
        ## cdef unsigned int n = fda_.size()
        ## cdef np.ndarray[np.float32_t, ndim=1] data
        ## data = np.zeros( (n,), dtype=np.float32)
        ## 
        ## cdef int i = 0
        ## while it != fda_.end():
        ##     data[i] = deref(it)
        ##     inc(it)
        ##     i += 1
        ## 
        ## return data
        ##

        # We see an improvement of ca 50x using the memory view method (and a
        # 2600x fold improvement compared to pure Python).

    def set_data(self, np.ndarray[float, ndim=1, mode="c"] data not None):
        """
        Sets the raw data for the float data array

        Example usage: 

          fd = pyopenms.FloatDataArray()
          data = numpy.array( [1, 2, 3, 5 ,6] ).astype(numpy.float32)
          fd.set_data(data)

        """
        cdef _FloatDataArray * fda_ = self.inst.get()
        fda_.clear()
        # Note: The np.ndarray[float, ndim=1, mode="c"] assures that we get a C-contiguous numpy array
        # See: https://github.com/cython/cython/wiki/tutorials-NumpyPointerToC

        # We use "assign" to directly to copy the numpy array (stored in
        # data.data) into the std::vector<float> in our FloatDataArray object
        cdef float * array_start = <float*>data.data
        cdef int N
        N = data.size
        fda_.assign(array_start, array_start + N)


        ## # Alternatively, we could also "push_back" each element:
        ## fda_.reserve(N)
        ## for i in range(N):
        ##     fda_.push_back(data[i])
        
        # We see an improvement of ca 10x using the assign method (and an 600x
        # improvement compared to pure Python).




from Types cimport *

from String cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/IsobaricQuantifierStatistics.h>" namespace "OpenMS":

    cdef cppclass IsobaricQuantifierStatistics "OpenMS::IsobaricQuantifierStatistics":
        IsobaricQuantifierStatistics() nogil except +
        IsobaricQuantifierStatistics(IsobaricQuantifierStatistics) nogil except +
        Size channel_count
        Size iso_number_ms2_negative
        Size iso_number_reporter_negative
        Size iso_number_reporter_different
        double iso_solution_different_intensity
        double iso_total_intensity_negative
        Size number_ms2_total
        Size number_ms2_empty
        # libcpp_map[ size_t, size_t ] empty_channels
        void reset() nogil except +



from Types cimport *
from Param cimport *
from AbsoluteQuantitationMethod cimport *
from AbsoluteQuantitationStandards cimport *
from Feature cimport *
from FeatureMap cimport *
from String cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/AbsoluteQuantitation.h>" namespace "OpenMS":

    cdef cppclass AbsoluteQuantitation(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler

        AbsoluteQuantitation() nogil except +
        AbsoluteQuantitation(AbsoluteQuantitation)  nogil except + #wrap-ignore

        void setQuantMethods(libcpp_vector[ AbsoluteQuantitationMethod ]& quant_methods) nogil except +
        libcpp_vector[ AbsoluteQuantitationMethod ] getQuantMethods() nogil except +
        double calculateRatio(Feature & component_1, Feature & component_2, const String & feature_name) nogil except +

        double applyCalibration(const Feature & component, const Feature & IS_component, const String & feature_name, 
                                const String & transformation_model, const Param & transformation_model_params) nogil except +
        void quantifyComponents(FeatureMap& unknowns) nogil except +

        bool optimizeCalibrationCurveIterative(
            libcpp_vector[ AQS_featureConcentration ] & component_concentrations,
            const String & feature_name,
            const String & transformation_model,
            const Param & transformation_model_params,
            Param & optimized_params) nogil except +

        void optimizeSingleCalibrationCurve(
            const String& component_name,
            libcpp_vector[AQS_featureConcentration]& component_concentrations) nogil except +

        # libcpp_map[ String, AbsoluteQuantitationMethod ] getQuantMethodsAsMap() nogil except +

        double calculateBias(double actual_concentration, double calculated_concentration) nogil except +

        Param fitCalibration(libcpp_vector[ AQS_featureConcentration ] & component_concentrations,
                             const String & feature_name,
                             const String & transformation_model,
                             Param transformation_model_params) nogil except +

        void calculateBiasAndR(libcpp_vector[ AQS_featureConcentration ] & component_concentrations,
                              const String & feature_name,
                              const String & transformation_model,
                              Param & transformation_model_params,
                              libcpp_vector[ double ] & biases,
                              double & correlation_coefficient) nogil except +

        # TODO: not implemented
        # void optimizeCalibrationCurves(libcpp_map[ String, libcpp_vector[ AQS_featureConcentration ] ] & components_concentrations) nogil except +
        



from ChromatogramPeak cimport *
from DefaultParamHandler cimport *
from Feature cimport *
from FeatureMap cimport *
from MSExperiment cimport *
from Peak1D cimport *
from ProgressLogger cimport *
from TransformationDescription cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/MapAlignmentAlgorithmPoseClustering.h>" namespace "OpenMS":

    cdef cppclass MapAlignmentAlgorithmPoseClustering(DefaultParamHandler, ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        MapAlignmentAlgorithmPoseClustering() nogil except +

        void align(FeatureMap,
                   TransformationDescription &
                   ) nogil except +

        void align(MSExperiment,
                   TransformationDescription &
                   ) nogil except +

        void setReference (FeatureMap) nogil except +
        void setReference (MSExperiment) nogil except +





from smart_ptr cimport shared_ptr
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/INTERFACES/DataStructures.h>" namespace "OpenMS::Interfaces":

  cdef cppclass BinaryDataArray:
        # here we misuse wrap-instances for renaming the instance, wrap-as is not supported for
        # classes, only for methods
        #
        # wrap-instances:
        #   _Interfaces_BinaryDataArray := BinaryDataArray
        BinaryDataArray() nogil except +
        BinaryDataArray(BinaryDataArray) nogil except +
        libcpp_vector[double] data

  ctypedef shared_ptr[BinaryDataArray] BinaryDataArrayPtr

  # See addons/Spectrum.pyx
  cdef cppclass Spectrum:
        # here we misuse wrap-instances for renaming the instance, wrap-as is not supported for
        # classes, only for methods
        #
        # wrap-instances:
        #   _Interfaces_Spectrum := Spectrum
        Spectrum() nogil except +
        Spectrum(Spectrum) nogil except +
        BinaryDataArrayPtr getMZArray() nogil except + #wrap-ignore
        BinaryDataArrayPtr getIntensityArray() nogil except + #wrap-ignore
        void setMZArray(BinaryDataArrayPtr data) nogil except + #wrap-ignore
        void setIntensityArray(BinaryDataArrayPtr data) nogil except + #wrap-ignore

  ctypedef shared_ptr[Spectrum] SpectrumPtr

  # See addons/Chromatogram.pyx
  cdef cppclass Chromatogram:
        # here we misuse wrap-instances for renaming the instance, wrap-as is not supported for
        # classes, only for methods
        #
        # wrap-instances:
        #   _Interfaces_Chromatogram := Chromatogram
        Chromatogram() nogil except +
        Chromatogram(Chromatogram) nogil except +
        BinaryDataArrayPtr getTimeArray() nogil except + #wrap-ignore
        BinaryDataArrayPtr getIntensityArray() nogil except + #wrap-ignore
        void setTimeArray(BinaryDataArrayPtr data) nogil except + #wrap-ignore
        void setIntensityArray(BinaryDataArrayPtr data) nogil except + #wrap-ignore

  ctypedef shared_ptr[Chromatogram] ChromatogramPtr




from Types cimport *
from OpenSwathDataStructures cimport *
from MSSpectrum cimport *
from MSChromatogram cimport *
from MSExperiment cimport *
from ISpectrumAccess cimport *

from SpectrumAccessOpenMS cimport *
from SpectrumAccessOpenMSCached cimport *
from SpectrumAccessQuadMZTransforming cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/DATAACCESS/SpectrumAccessOpenMSInMemory.h>" namespace "OpenMS":
    
    cdef cppclass SpectrumAccessOpenMSInMemory(ISpectrumAccess) :
        # wrap-inherits:
        #  ISpectrumAccess

        SpectrumAccessOpenMSInMemory() # wrap-pass-constructor

        # SpectrumAccessOpenMSInMemory(ISpectrumAccess & origin) nogil except +
        SpectrumAccessOpenMSInMemory(SpectrumAccessOpenMS) nogil except +
        SpectrumAccessOpenMSInMemory(SpectrumAccessOpenMSCached) nogil except +
        SpectrumAccessOpenMSInMemory(SpectrumAccessOpenMSInMemory) nogil except +
        SpectrumAccessOpenMSInMemory(SpectrumAccessQuadMZTransforming) nogil except +




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from MRMTransitionGroup cimport *
from MRMFeature cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from MSChromatogram cimport *
from ChromatogramPeak cimport *
from SignalToNoiseEstimatorMedian cimport *
from SavitzkyGolayFilter cimport *
from GaussFilter cimport *
from LinearResampler cimport *
from PeakPickerHiRes cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMTransitionGroupPicker.h>" namespace "OpenMS":
    
    cdef cppclass MRMTransitionGroupPicker(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        MRMTransitionGroupPicker() nogil except +
        MRMTransitionGroupPicker(MRMTransitionGroupPicker) nogil except + #wrap-ignore

        void pickTransitionGroup(MRMTransitionGroup[MSChromatogram, LightTransition] transition_group) nogil except +
        void pickTransitionGroup(MRMTransitionGroup[MSChromatogram, ReactionMonitoringTransition] transition_group) nogil except +

        MRMFeature createMRMFeature(MRMTransitionGroup[ MSChromatogram, LightTransition] transition_group,
                                    libcpp_vector[ MSChromatogram ] & picked_chroms,
                                    libcpp_vector[ MSChromatogram ] & smoothed_chroms,
                                    const int chr_idx,
                                    const int peak_idx) nogil except +

        void remove_overlapping_features(libcpp_vector[ MSChromatogram ] & picked_chroms,
                                         double best_left, 
                                         double best_right) nogil except +

        void findLargestPeak(libcpp_vector[ MSChromatogram ] & picked_chroms, int & chr_idx, int & peak_idx) nogil except +

        void findWidestPeakIndices(libcpp_vector[ MSChromatogram ] & picked_chroms, Int & chrom_idx, Int & point_idx) nogil except +



import unittest
import os

import pyopenms

class TestMSSpectrumAndRichSpectrum(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.abspath(__file__))


    def testMSSpectrum(self):
        spec = pyopenms.MSSpectrum()
        p = pyopenms.Peak1D()
        p.setMZ(500.0)
        p.setIntensity(1e5)
        spec.push_back(p)

        p_back, = list(spec)
        assert isinstance(p_back, pyopenms.Peak1D)
        assert p_back.getMZ() == 500.0
        assert p_back.getIntensity() == 1e5

        spec.updateRanges()
        assert isinstance(spec.getMin()[0], float)
        assert isinstance(spec.getMax()[0], float)
        assert isinstance(spec.getMinInt(), float)
        assert isinstance(spec.getMaxInt(), float)

        assert spec.getMaxInt() == 1e5
        assert spec.getMinInt() == 1e5

if __name__ == '__main__':
    unittest.main()



from String cimport *
from Software cimport *
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/IonDetector.h>" namespace "OpenMS":

    cdef cppclass IonDetector(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface

        IonDetector() nogil except +
        IonDetector(IonDetector) nogil except + # wrap-ignore

        # returns the detector type
        Type_IonDetector getType() nogil except +
        # sets the detector type
        void setType(Type_IonDetector type_) nogil except +

        # returns the acquisition mode
        AcquisitionMode getAcquisitionMode() nogil except +
        # sets the acquisition mode
        void setAcquisitionMode(AcquisitionMode acquisition_mode) nogil except +

        # returns the resolution (in ns)
        double getResolution() nogil except +
        # sets the resolution (in ns)
        void setResolution(double resolution) nogil except +

        # returns the analog-to-digital converter sampling frequency (in Hz)
        double getADCSamplingFrequency() nogil except +
        # sets the analog-to-digital converter sampling frequency (in Hz)
        void setADCSamplingFrequency(double ADC_sampling_frequency) nogil except +

        Int getOrder() nogil except +
        # sets the order
        void setOrder(Int order) nogil except +

cdef extern from "<OpenMS/METADATA/IonDetector.h>" namespace "OpenMS::IonDetector":

        # Detector type
        cdef enum Type_IonDetector "OpenMS::IonDetector::Type":
          # wrap-attach:
          #     IonDetector
          TYPENULL,                                                                 #< Unknown
          ELECTRONMULTIPLIER,                                           #< Electron multiplier
          PHOTOMULTIPLIER,                                                  #< Photo multiplier
          FOCALPLANEARRAY,                                                  #< Focal plane array
          FARADAYCUP,                                                           #< Faraday cup
          CONVERSIONDYNODEELECTRONMULTIPLIER,           #< Conversion dynode electron multiplier
          CONVERSIONDYNODEPHOTOMULTIPLIER,                  #< Conversion dynode photo multiplier
          MULTICOLLECTOR,                                                   #< Multi-collector
          CHANNELELECTRONMULTIPLIER,                            #< Channel electron multiplier
          CHANNELTRON,                                                          #< channeltron
          DALYDETECTOR,                                                         #< daly detector
          MICROCHANNELPLATEDETECTOR,                            #< microchannel plate detector
          ARRAYDETECTOR,                                                    #< array detector
          CONVERSIONDYNODE,                                                 #< conversion dynode
          DYNODE,                                                                   #< dynode
          FOCALPLANECOLLECTOR,                                          #< focal plane collector
          IONTOPHOTONDETECTOR,                                          #< ion-to-photon detector
          POINTCOLLECTOR,                                                   #< point collector
          POSTACCELERATIONDETECTOR,                                 #< postacceleration detector
          PHOTODIODEARRAYDETECTOR,                                  #< photodiode array detector
          INDUCTIVEDETECTOR,                                            #< inductive detector
          ELECTRONMULTIPLIERTUBE,                                   #< electron multiplier tube
          SIZE_OF_TYPE

        # Acquisition mode
        cdef enum AcquisitionMode:
          # wrap-attach:
          #     IonDetector
          ACQMODENULL,                          #< Unknown
          PULSECOUNTING,                    #< Pulse counting
          ADC,                                          #< Analog-digital converter
          TDC,                                          #< Time-digital converter
          TRANSIENTRECORDER,            #< Transient recorder
          SIZE_OF_ACQUISITIONMODE



from Matrix cimport *


# continue with extra code if needed

    def get_matrix(self):
        """Cython signature: numpy_matrix get_matrix()
        """

        cdef _Matrix[double] * mat_ = self.inst.get()

        cdef unsigned int rows = mat_.rows()
        cdef unsigned int cols = mat_.cols()

        cdef libcpp_vector[double] tmp_vec;
        tmp_vec = mat_.asVector();

        xarr = np.asarray(tmp_vec)
        xarr = xarr.reshape(rows, cols)
        return xarr

    def get_matrix_as_view(self):
        """Cython signature: numpy_matrix get_matrix()
        """

        cdef _Matrix[double] * mat_ = self.inst.get()

        cdef unsigned int rows = mat_.rows()
        cdef unsigned int cols = mat_.cols()
        cdef unsigned int n = rows * cols
        cdef np.ndarray[double, ndim=2] data
        data = np.zeros( (rows,cols), dtype=np.float64)

        cdef libcpp_vector[double] * vec_ptr = <libcpp_vector[double]*> mat_
        cdef double * raw_ptr =  address(deref(vec_ptr)[0]) 

        ## # We use a memory view to get the data from the raw data
        ## # See https://cython.readthedocs.io/en/latest/src/userguide/memoryviews.html 
        ## # See https://stackoverflow.com/questions/43021574/cast-c-array-into-numpy-array-cython-typed-memoryview-in-cython-code
        cdef double[:] vec_view = <double[:n]>raw_ptr # cast to memoryview, refer to the underlying buffer without copy
        xarr = np.asarray(vec_view) # numpy array refer to the underlying buffer without copy
        xarr = xarr.reshape(rows, cols)
        return xarr

    def set_matrix(self, np.ndarray[double, ndim=2, mode="c"] data not None):
        """Cython signature: numpy_matrix set_matrix()
        """

        cdef _Matrix[double] * mat_ = self.inst.get()

        cdef unsigned int rows = data.shape[0]
        cdef unsigned int cols = data.shape[1]
        mat_.resize(rows, cols, 0)

        cdef int i = 0
        cdef int j = 0
        for i in range(int(rows)):
            for j in range(int(cols)):
                mat_.setValue(i,j,data[i][j])






from Types cimport *
from DefaultParamHandler cimport *
from MSSpectrum cimport *
from TheoreticalSpectrumGenerator cimport *
from SpectrumAlignment cimport *
from Peak1D cimport *
from PeptideHit cimport *
from PeptideIdentification cimport *

cdef extern from "<OpenMS/CHEMISTRY/SpectrumAnnotator.h>" namespace "OpenMS":
    
    cdef cppclass SpectrumAnnotator(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler

        SpectrumAnnotator() nogil except +
        SpectrumAnnotator(SpectrumAnnotator) nogil except +

        void annotateMatches(MSSpectrum & spec, PeptideHit & ph, TheoreticalSpectrumGenerator & tg, SpectrumAlignment & sa) nogil except +
        void addIonMatchStatistics(PeptideIdentification & pi, MSSpectrum & spec, TheoreticalSpectrumGenerator & tg, SpectrumAlignment & sa) nogil except +




from Types cimport *
from OPXLDataStructs cimport *
from AASequence cimport *

cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLDataStructs.h>" namespace "OpenMS":

    cdef cppclass AASeqWithMass "OpenMS::OPXLDataStructs::AASeqWithMass":

        AASeqWithMass(AASeqWithMass) nogil except +
        AASeqWithMass() nogil except +

        double peptide_mass
        AASequence peptide_seq
        PeptidePosition position



from DefaultParamHandler cimport *
from SimTypes cimport *
from FeatureMap cimport *
from Feature cimport *

cdef extern from "<OpenMS/SIMULATION/DigestSimulation.h>" namespace "OpenMS":
    
    cdef cppclass DigestSimulation "OpenMS::DigestSimulation":
        DigestSimulation() nogil except +
        DigestSimulation(DigestSimulation) nogil except +
        void digest(FeatureMap & feature_map) nogil except +




from Types cimport *
from ProgressLogger cimport *
from String cimport *
from TextFile cimport *
from File cimport *

cdef extern from "<OpenMS/ANALYSIS/SVM/SVMWrapper.h>" namespace "OpenMS":
    
    cdef cppclass SVMWrapper "OpenMS::SVMWrapper":
        SVMWrapper() nogil except +
        SVMWrapper(SVMWrapper) nogil except + #wrap-ignore

        void setParameter(SVM_parameter_type type_, Int value) nogil except +
        void setParameter(SVM_parameter_type type_, double value) nogil except +
        # Int train(struct svm_problem *problem) nogil except +
        Int train(SVMData &problem) nogil except +
        void saveModel(String modelFilename) nogil except +
        void loadModel(String modelFilename) nogil except +
        # void predict(struct svm_problem *problem, libcpp_vector[ double ] &predicted_labels) nogil except +
        void predict(SVMData &problem, libcpp_vector[ double ] &results) nogil except +
        Int getIntParameter(SVM_parameter_type type_) nogil except +
        double getDoubleParameter(SVM_parameter_type type_) nogil except +
        # TODO STL map with wrapped key
        # void predict(libcpp_vector[ svm_node * ] &vectors, libcpp_vector[ double ] &predicted_rts) nogil except +
        # double performCrossValidation(svm_problem *problem_ul, SVMData &problem_l, bool is_labeled, libcpp_map[ SVM_parameter_type, double ] &start_values_map, libcpp_map[ SVM_parameter_type, double ] &step_sizes_map, libcpp_map[ SVM_parameter_type, double ] &end_values_map, Size number_of_partitions, Size number_of_runs, libcpp_map[ SVM_parameter_type, double ] &best_parameters, bool additive_step_sizesrue, bool outputalse, String, bool mcc_as_performance_measurealse) nogil except +
        double getSVRProbability() nogil except +
        # void getSignificanceBorders(svm_problem *data, libcpp_pair[ double, double ] &borders, double confidence.95, Size number_of_runs, Size number_of_partitions, double step_size.01, Size max_iterations000000) nogil except +
        void getSignificanceBorders(SVMData &data, libcpp_pair[ double, double ] &sigmas, double confidence, Size number_of_runs, Size number_of_partitions, double step_size, Size max_iterations) nogil except +
        double getPValue(double sigma1, double sigma2, libcpp_pair[ double, double ] point) nogil except +
        # void getDecisionValues(svm_problem *data, libcpp_vector[ double ] &decision_values) nogil except +
        # void scaleData(svm_problem *data, Int max_scale_value1) nogil except +
        # svm_problem * computeKernelMatrix(svm_problem *problem1, svm_problem *problem2) nogil except +
        # svm_problem * computeKernelMatrix(SVMData &problem1, SVMData &problem2) nogil except +
        # void setTrainingSample(svm_problem *training_sample) nogil except +
        void setTrainingSample(SVMData &training_sample) nogil except +
        # void getSVCProbabilities(struct svm_problem *problem, libcpp_vector[ double ] &probabilities, libcpp_vector[ double ] &prediction_labels) nogil except +
        void setWeights(libcpp_vector[ int ] &weight_labels, libcpp_vector[ double ] &weights) nogil except +
        # void createRandomPartitions(svm_problem *problem, Size number, libcpp_vector[ svm_problem * ] &partitions) nogil except +
        void createRandomPartitions(SVMData &problem, Size number, libcpp_vector[ SVMData ] &problems) nogil except +
        # TODO: Mismatch between C++ return type ([u'svm_problem *']) and Python return type (['void']) in function public mergePartitions:
        # svm_problem * mergePartitions(libcpp_vector[ svm_problem * ] &problems, Size except) nogil except +
        void mergePartitions(libcpp_vector[ SVMData ] &problems, Size except_, SVMData &merged_problem) nogil except +
        # void getLabels(svm_problem *problem, libcpp_vector[ double ] &labels) nogil except +
        # double kernelOligo(libcpp_vector[ libcpp_pair[ int, double ] ] &x, libcpp_vector[ libcpp_pair[ int, double ] ] &y, libcpp_vector[ double ] &gauss_table, int max_distance1) nogil except +
        # double kernelOligo(svm_node *x, svm_node *y, libcpp_vector[ double ] &gauss_table, double sigma_square, Size max_distance0) nogil except +
        void calculateGaussTable(Size border_length, double sigma, libcpp_vector[ double ] &gauss_table) nogil except +

cdef extern from "<OpenMS/ANALYSIS/SVM/SVMWrapper.h>" namespace "OpenMS":
    
    cdef cppclass SVMData "OpenMS::SVMData":
        SVMData() nogil except +
        SVMData(SVMData) nogil except + #wrap-ignore

        # TODO nested STL
        # libcpp_vector[ libcpp_vector[ libcpp_pair[ Int, double ] ] ] sequences
        libcpp_vector[ double ] labels
        # TODO nested STL
        # SVMData(libcpp_vector[ libcpp_vector[ libcpp_pair[ Int, double ] ] ] &seqs, libcpp_vector[ double ] &lbls) nogil except +
        bool operator==(SVMData &rhs) nogil except +
        bool store(const String &filename) nogil except +
        bool load(const String &filename) nogil except +


cdef extern from "<OpenMS/ANALYSIS/SVM/SVMWrapper.h>" namespace "OpenMS::SVMWrapper":
    cdef enum SVM_parameter_type "OpenMS::SVMWrapper::SVM_parameter_type":
        #wrap-attach:
        #    SVMWrapper
        SVM_TYPE
        KERNEL_TYPE
        DEGREE
        C
        NU
        P
        GAMMA
        PROBABILITY
        SIGMA
        BORDER_LENGTH

cdef extern from "<OpenMS/ANALYSIS/SVM/SVMWrapper.h>" namespace "OpenMS::SVMWrapper":
    cdef enum SVM_kernel_type "OpenMS::SVMWrapper::SVM_kernel_type":
        #wrap-attach:
        #    SVMWrapper
        OLIGO
        OLIGO_COMBINED




from Feature cimport *
from FeatureMap cimport *
from String cimport *
from FeatureFileOptions cimport *

cdef extern from "<OpenMS/FORMAT/FeatureXMLFile.h>" namespace "OpenMS":

    cdef cppclass FeatureXMLFile:
        FeatureXMLFile() nogil except +

        void load(String, FeatureMap &) nogil except+
        void store(String, FeatureMap &) nogil except+

        FeatureFileOptions getOptions() nogil except +
        void setOptions(FeatureFileOptions) nogil except +

        Size loadSize(String path) nogil except +







    def map2csv(self, dict csv_table, String separator):
        cdef libcpp_map[_String, libcpp_map[_String, _String] ] c_dict_outer
        cdef libcpp_map[_String, _String] c_dict_inner
        for k,v in csv_table.iteritems():
            c_dict_inner.clear()
            for k_i,v_i in v.iteritems():
                c_dict_inner[ _String(<char *>k_i) ] = _String(<char *>v_i)
            c_dict_outer[ _String(<char *>k) ] = c_dict_inner

        
        cdef _String _r = self.inst.get().map2csv(c_dict_outer, deref(separator.inst.get()) )
        py_result = _cast_const_away(<char*>_r.c_str())
        return py_result




from Types cimport *
from ConsensusIDAlgorithm cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/ConsensusIDAlgorithmIdentity.h>" namespace "OpenMS":
    
    cdef cppclass ConsensusIDAlgorithmIdentity(ConsensusIDAlgorithm) :
        # wrap-inherits:
        #  ConsensusIDAlgorithm
        # wrap-ignore
        # ABSTRACT class
        # no-pxd-import
        ConsensusIDAlgorithmIdentity() nogil except + #wrap-ignore
        ConsensusIDAlgorithmIdentity(ConsensusIDAlgorithmIdentity) nogil except + #wrap-ignore




from Types cimport *
from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/DATASTRUCTURES/Matrix.h>" namespace "OpenMS":
    
    cdef cppclass Matrix[ValueT]:
        # wrap-instances:
        #   MatrixDouble := Matrix[double]

        Matrix() nogil except +
        Matrix(Matrix[ValueT]) nogil except +
        Matrix(size_t rows, size_t cols, ValueT value) nogil except +
        # const_reference operator()(size_t i, size_t j) nogil except +
        # reference operator()(size_t i, size_t j) nogil except +
        # const_reference getValue(size_t i, size_t j) nogil except +
        ValueT getValue(size_t i, size_t j) nogil
        void setValue(size_t i, size_t j, ValueT value) nogil
        ## The following two lines introduce an odd bug:
        # static PyObject *__pyx_convert_vector_to_py_double is declared twice by Cython:
        # TODO look into Cython Bug
        # libcpp_vector[ValueT] row(size_t i) nogil except +
        # libcpp_vector[ValueT] col(size_t i) nogil except +
        libcpp_vector[ValueT] asVector() nogil except + # wrap-ignore
        void clear() nogil except +
        void resize(size_t i, size_t j, ValueT value) nogil except +
        void resize(libcpp_pair[ size_t, size_t ] & size_pair, ValueT value) nogil except +
        size_t rows() nogil
        size_t cols() nogil
        libcpp_pair[ size_t, size_t ] sizePair() nogil except +
        size_t index(size_t row, size_t col) nogil except +
        libcpp_pair[ size_t, size_t ] indexPair(size_t index) nogil except +
        size_t colIndex(size_t index) nogil except +
        size_t rowIndex(size_t index) nogil except +
        ## bool operator==(Matrix & rhs) nogil except +
        ## bool operator<(Matrix & rhs) nogil except +
        # TEMPLATE # void setMatrix(ValueType matrix) nogil except +
        # TEMPLATE # # NAMESPACE # std::ostream  operator<[(std::ostream & os, Matrix[ Value ] & matrix) nogil except +
        #   MatrixUnsignedInt := Matrix[unsigned int]




from Types cimport *
from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair
from libcpp.set cimport set as libcpp_set
from libcpp.vector cimport vector as libcpp_vector
from FeatureMap cimport *
from MSExperiment cimport *
from IDMapper cimport *
from FeatureXMLFile cimport *
# from PSLPFormulation cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *
from LPWrapper cimport *

cdef extern from "<OpenMS/ANALYSIS/TARGETED/OfflinePrecursorIonSelection.h>" namespace "OpenMS":
    
    cdef cppclass OfflinePrecursorIonSelection(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        OfflinePrecursorIonSelection() nogil except +
        OfflinePrecursorIonSelection(OfflinePrecursorIonSelection) nogil except + #wrap-ignore

        void makePrecursorSelectionForKnownLCMSMap(FeatureMap & features,
                                                   MSExperiment & experiment,
                                                   MSExperiment & ms2,
                                                   libcpp_set[ int ] & charges_set, bool
                                                   feature_based) nogil except +

        # TODO nested STL
        void getMassRanges(FeatureMap & features, 
                           MSExperiment & experiment,
                           libcpp_vector[ libcpp_vector[ libcpp_pair[ Size, Size ] ] ] & indices) nogil except + # wrap-ignore

        void createProteinSequenceBasedLPInclusionList(String include_, String rt_model_file, String pt_model_file, FeatureMap & precursors) nogil except +
        void setLPSolver(SOLVER solver) nogil except +
        SOLVER getLPSolver() nogil except +




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from DRange cimport *
from MSNumpressCoder cimport *

cdef extern from "<OpenMS/FORMAT/OPTIONS/PeakFileOptions.h>" namespace "OpenMS":

    cdef cppclass PeakFileOptions:

        PeakFileOptions() nogil except +
        PeakFileOptions(PeakFileOptions) nogil except +

        void setMetadataOnly(bool) nogil except +
        bool getMetadataOnly()     nogil except +

        void setWriteSupplementalData(bool) nogil except +
        bool getWriteSupplementalData()     nogil except +

        void setMSLevels(libcpp_vector[int] levels) nogil except +
        void addMSLevel(Int level) nogil except +
        void clearMSLevels()       nogil except +
        bool hasMSLevels()         nogil except +
        bool containsMSLevel(int level)  nogil except +
        libcpp_vector[int] getMSLevels()    nogil except +

        void setCompression(bool) nogil except +
        bool getCompression()     nogil except +

        void setMz32Bit(bool mz_32_bit) nogil except +
        bool getMz32Bit() nogil except +
        void setIntensity32Bit(bool int_32_bit) nogil except +
        bool getIntensity32Bit() nogil except +

        void setRTRange(DRange1 & range_) nogil except +
        bool hasRTRange() nogil except +
        DRange1 getRTRange() nogil except +
        void setMZRange(DRange1 & range_) nogil except +
        bool hasMZRange() nogil except +
        DRange1 getMZRange() nogil except +
        void setIntensityRange(DRange1 & range_) nogil except +
        bool hasIntensityRange() nogil except +
        DRange1 getIntensityRange() nogil except +

        Size getMaxDataPoolSize() nogil except +
        void setMaxDataPoolSize(Size s) nogil except +

        void setSortSpectraByMZ(bool doSort) nogil except +
        bool getSortSpectraByMZ() nogil except +
        void setSortChromatogramsByRT(bool doSort) nogil except +
        bool getSortChromatogramsByRT() nogil except +

        bool hasFilters() nogil except +

        void setFillData(bool only) nogil except +
        bool getFillData() nogil except +

        void setSkipXMLChecks(bool only) nogil except +
        bool getSkipXMLChecks() nogil except +

        bool getWriteIndex() nogil except +
        void setWriteIndex(bool write_index) nogil except +

        NumpressConfig getNumpressConfigurationMassTime() nogil except +
        void setNumpressConfigurationMassTime(NumpressConfig config) nogil except +

        NumpressConfig getNumpressConfigurationIntensity() nogil except +
        void setNumpressConfigurationIntensity(NumpressConfig config) nogil except +

        NumpressConfig getNumpressConfigurationFloatDataArray() nogil except +
        void setNumpressConfigurationFloatDataArray(NumpressConfig config) nogil except +

        void setForceMQCompatability(bool forceMQ) nogil except +
        bool getForceMQCompatability() nogil except +

        void setForceTPPCompatability(bool forceTPP) nogil except +
        bool getForceTPPCompatability() nogil except +
        



from Types cimport *
from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

from MassTrace cimport *

from DefaultParamHandler cimport *
from ProgressLogger cimport *

from libcpp.vector cimport vector as libcpp_vector

cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/MassTraceDetection.h>" namespace "OpenMS":

    cdef cppclass MassTraceDetection(ProgressLogger, DefaultParamHandler):
        # wrap-inherits:
        #    ProgressLogger
        #    DefaultParamHandler

        MassTraceDetection()      nogil except +

        void run(MSExperiment & input_map,
                libcpp_vector[Kernel_MassTrace] & traces,
                Size max_traces
                ) nogil except +




from Types cimport *
from String cimport *

cdef extern from "<OpenMS/METADATA/SourceFile.h>" namespace "OpenMS":

    cdef cppclass SourceFile:
        SourceFile() nogil except +
        SourceFile(SourceFile) nogil except +
        String getNameOfFile() nogil except +
        void setNameOfFile(String) nogil except +

        String getPathToFile() nogil except +
        void setPathToFile(String) nogil except +

        float getFileSize() nogil except +
        void setFileSize(float) nogil except +

        String getFileType() nogil except +
        void setFileType(String) nogil except +

        String getChecksum() nogil except +
        void setChecksum(String, ChecksumType) nogil except +

        ChecksumType getChecksumType() nogil except +

        String getNativeIDType() nogil except +
        void setNativeIDType(String) nogil except +

        String getNativeIDTypeAccession() nogil except +
        void setNativeIDTypeAccession(const String & accesssion) nogil except +

cdef extern from "<OpenMS/METADATA/SourceFile.h>" namespace "OpenMS::SourceFile":
    cdef enum ChecksumType:
           UNKNOWN_CHECKSUM, SHA1, MD5, SIZE_OF_CHECKSUMTYPE



import os
import glob
import re
pxd_files = glob.glob("../pxds/*.pxd")
classdocu_base = "http://ftp.mi.fu-berlin.de/OpenMS/release-documentation/html"
classdocu_base = "http://www.openms.de/current_doxygen/html/"

# Variables:
# after how many characters should a line be wrapped
Linebreak = 85
# Whether to remove comments and "nogil" statements
remove_nogil = True
remove_comments = True
ignores = ["ctime", "smart_ptr", "Time"]

# Regexes to determine if a line is comment only or empty
comment_line = re.compile("^\s*#")
empty_line = re.compile("^\s*$")

# Some state variables to keep track of 
#  - whether the previous line was empty or not
#  - whether the current code block contains any pyOpenMS specific "wrap-XXX" statements
# 
# We want to remove all comments except those pertaining to autowrap (Python
# wrapping) directives as they are important to understand the code
prevline_empty = False
wrap_found = False

# Apply line break
def line_break(line):
    # print "trying to break line ", line
    if len(line) < Linebreak:return line

    for i in reversed(range(Linebreak)):
        if line[i] == " ":
            break
    if len(line) - i < 9: return line
    if i == 0:
        print "Could not break line", line
        # return line
        return line[:Linebreak] + "\\\n" + " "*6 + line_break(line[Linebreak:])
    # print "call again with i", i, "and len ", len(line)
    return line[:i] + "\\\n" + " "*6 + line_break(line[i:])

# Prepare a line for printing
def prepare_line(line):
    global wrap_found, prevline_empty
    # replace the indent from 4/8 to 2/4 -> get more space
    line = line.replace("        ", "xxxx")
    line = line.replace("    ", "  ")
    line = line.replace("xxxx", "    ")
    if remove_nogil:
        line = line.replace("nogil except +", "")

    # reset
    if prevline_empty:
        wrap_found = False

    if remove_comments:
        if line.find("wrap-") != -1:
            wrap_found = True
        if re.search(comment_line, line) is not None and not wrap_found:
            if line.find("COMMENT:") != -1:
                line = line.replace("COMMENT:", "")
            elif line.find(" COMMENT:") != -1:
                line = line.replace(" COMMENT:", "")
            else:
                print "Remove line", line.strip()
                return ""
    return line

def get_namespace(pxd):
    filehandle = open(pxd)
    fulltext = filehandle.read()
    filehandle.close()
    import re
    match = re.search("cdef extern.*?namespace\s*\"([^\"]*)\"", fulltext)
    if not match:
        return "OpenMS"
    else:
        return match.group(1)


def get_header(title, ns):

    return r"""
$\rightarrow$ \textit{\href{%s/class%s_1_1%s.html}{Link
to OpenMS documentation}}

Wrapped functions in Python:
""" % (classdocu_base, ns, title)

write_handle = open("appendix.tex", "w")
for pxd in sorted(pxd_files):

    # get title for latex section
    wrap_found = False
    title = pxd[8:-4]
    if title in ignores:
        continue

    title = title.replace("_", "\\_")
    write_handle.write("\subsection{%s}\n\n" % title)
    write_handle.write("\label{%s}\n\n" % title)
    # write_handle.write("{\\tiny\n    \\begin{verbatim}")

    # Write the header (link to OpenMS documentation)
    write_handle.write( get_header(title, get_namespace(pxd)) )
    write_handle.write("{    \\begin{verbatim}")

    # Start writing the PDX file content
    filehandle = open(pxd)
    found_start = False
    for line in filehandle:
        if line.find("import") == -1 and len(line.strip() ) > 0:
            found_start = True
        if line.find("import") == -1 and found_start:
            # print line.strip()
            line = prepare_line(line)
            line = line_break(line)

            # do not print two empty lines in a row
            if re.search(empty_line, line) is not None:
                if prevline_empty:
                    continue
                prevline_empty = True
            else:
                prevline_empty = False

            write_handle.write(line)

    write_handle.write("\end{verbatim}\n}\n\n")

write_handle.close()






from libcpp cimport *
from Types cimport *
from DPosition cimport *

cdef extern from "<OpenMS/KERNEL/Peak1D.h>" namespace "OpenMS":

    cdef cppclass Peak1D:
        Peak1D() nogil except +
        Peak1D(Peak1D &) nogil except +

        # We will not catch C++ exceptions for get/set methods for performance
        # reasons (no memory allocation is involved).
        float getIntensity() nogil 
        double getMZ() nogil 
        void setMZ(double) nogil 
        void setIntensity(float) nogil 
        bool operator==(Peak1D) nogil except +
        bool operator!=(Peak1D) nogil except +
        double getPos() nogil 
        void setPos(double pos) nogil 
        # DPosition1 getPosition() nogil except + # wrap-ignore
        # void setPosition(DPosition1 position) nogil except + # wrap-ignore
    



from Types cimport *
from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/ID/IdentificationData.h>" namespace "OpenMS":
    
    cdef cppclass IdentificationData(MetaInfoInterface) :
        # wrap-inherits:
        #  MetaInfoInterface

        IdentificationData() nogil except +



from Types cimport *
from IntegerMassDecomposer cimport *

# Cannot use unsigned int as template argument ?
# ctypedef libcpp_map[unsigned int, libcpp_pair[unsigned int, unsigned int] ] constraints_type
ctypedef unsigned long long number_of_decompositions_type

cdef extern from "<OpenMS/CHEMISTRY/MASSDECOMPOSITION/IMS/RealMassDecomposer.h>" namespace "OpenMS::ims":
    
    cdef cppclass RealMassDecomposer "OpenMS::ims::RealMassDecomposer":
        RealMassDecomposer() nogil except + #wrap-ignore
        RealMassDecomposer(RealMassDecomposer) nogil except + #wrap-ignore
        RealMassDecomposer(IMSWeights & weights) nogil except +
        # libcpp_vector[ libcpp_vector[unsigned int] ] getDecompositions(double mass, double error) nogil except +
        # libcpp_vector[int] getDecompositions(double mass, double error, constraints_type & constraints) nogil except +
        unsigned long long getNumberOfDecompositions(double mass, double error) nogil except +




from MSExperiment  cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from String cimport *
from ProgressLogger cimport *
from ISpectrumAccess cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/DATAACCESS/SimpleOpenMSSpectraAccessFactory.h>" namespace "OpenMS":

    # see ./pyOpenMS/pyopenms/python_extras.py file 
    cdef cppclass SimpleOpenMSSpectraFactory(ProgressLogger):
        # wrap-ignore
        # no-pxd-import

        SimpleOpenMSSpectraFactory() nogil except +

        # shared_ptr[ISpectrumAccess] getSpectrumAccessOpenMSPtr(MSExperiment exp) # wrap-ignore
        # OPENSWATHALGO_DLLAPI typedef boost::shared_ptr<ISpectrumAccess> SpectrumAccessPtr;






def plotChromatogram(c):
    import pylab
    x, y = c.get_peaks()
    pylab.plot( x, y )
    pylab.xlabel("Retention time")
    pylab.ylabel("Intensity")
    pylab.show()

def plotSpectrum(s):
    import pylab
    x, y = s.get_peaks()
    pylab.bar( x, y )
    pylab.xlabel("m/z")
    pylab.ylabel("Intensity")
    pylab.show()



from libcpp.vector cimport vector as libcpp_vector
from ConsensusMap cimport *
from DefaultParamHandler cimport *
from Feature cimport *
from FeatureMap cimport *
from String cimport *

from ProteinIdentification cimport *
from PeptideIdentification cimport *

from PeptideHit cimport *

from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/AScore.h>" namespace "OpenMS":

    cdef cppclass AScore(DefaultParamHandler):
        # wrap-inherits:
        #  DefaultParamHandler
        AScore() nogil except +
        AScore(AScore) nogil except + # wrap-ignore

        PeptideHit compute(PeptideHit & hit,
                           MSSpectrum & real_spectrum) nogil except +


    cdef cppclass ProbablePhosphoSites:

        ProbablePhosphoSites() nogil except +
        ProbablePhosphoSites(ProbablePhosphoSites) nogil except + # wrap-ignore

        Size first
        Size second
        Size seq_1
        Size seq_2
        Size peak_depth
        Size AScore







    def registerExperiment(self, MSExperiment exp, list labels):

        assert isinstance(labels, list) and all(isinstance(li, list) and all( 
                isinstance(inner_li, list) and all(
                    isinstance(inner_pair, list) and len(inner_pair) == 2 and
                        isinstance(inner_pair[0], String) and isinstance(inner_pair[1], float)
                for inner_pair in inner_li)
            for inner_li in li) 
        for li in labels), 'arg proteins wrong type'

        cdef libcpp_vector[libcpp_vector[libcpp_pair[_String, double]]] * v0 = new libcpp_vector[libcpp_vector[libcpp_pair[_String, double]]]()
        cdef libcpp_vector[libcpp_pair[_String, double] ] * v0_rec = new libcpp_vector[libcpp_pair[_String, double]]()
        cdef String innerstring_
        cdef libcpp_vector[libcpp_pair[_String, double] ].iterator it_labels_rec
        cdef libcpp_pair[_String, double] * aPair
        for labels_rec in labels:
            v0_rec.clear()
            for item0_rec in labels_rec:
                
                # Build new pair, push it, delete it 
                innerstring_ = item0_rec.first
                aPair = new libcpp_pair[ _String, double]( deref(innerstring_.inst.get()), <double>item0_rec.second )
                v0_rec.push_back(deref(aPair))
                del aPair

            v0.push_back(deref(v0_rec))

        self.inst.get().registerExperiment( (deref( exp.inst.get() ))  , (deref(v0)) )

        del v0
        del v0_rec

    # def getRatios(self):
    #     cdef libcpp_map[_String, _Ratio] res 
    #     c_res = self.inst.get().getRatios()
    #     #
    #     ## Get the data back from C++
    #     #
    #     replace = dict()
    #     cdef libcpp_map[_String, _Ratio].iterator it_ripped = c_res.begin()
    #     cdef _Ratio myRatio
    #     cdef Ratio py_ratio
    #     while it_ripped != c_res.end():
    #         
    #         myRatio = deref(it_ripped).second
    #         py_ratio = Ratio.__new__(Ratio)
    #         py_ratio.inst = shared_ptr[_Ratio](new _Ratio(myRatio))

    #         replace[ <libcpp_string>deref(it_ripped).first ]  = py_ratio
    #         inc(it_ripped)

    #     return replace






from MSSpectrum cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from Param cimport *
from DefaultParamHandler cimport *
from ExperimentalSettings cimport *
from SwathMap cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/SwathFileConsumer.h>" namespace "OpenMS":

    cdef cppclass FullSwathFileConsumer:
        #wrap-ignore
        #no-pxd-import

        FullSwathFileConsumer() nogil except + #wrap-ignore
        FullSwathFileConsumer(FullSwathFileConsumer) nogil except + #wrap-ignore
        FullSwathFileConsumer(libcpp_vector[SwathMap] swath_boundaries) nogil except +

        void setExpectedSize(Size s, Size c) nogil except +
        void setExperimentalSettings(ExperimentalSettings exp) nogil except +

        void retrieveSwathMaps(libcpp_vector[SwathMap]& maps) nogil except +

        void consumeSpectrum(MSSpectrum & s) nogil except +
        void consumeChromatogram(MSChromatogram & c) nogil except +

cdef extern from "<OpenMS/FORMAT/DATAACCESS/SwathFileConsumer.h>" namespace "OpenMS":

    cdef cppclass RegularSwathFileConsumer(FullSwathFileConsumer):
        # wrap-inherits:
        #    FullSwathFileConsumer

        RegularSwathFileConsumer() nogil except +
        RegularSwathFileConsumer(RegularSwathFileConsumer &) nogil except + # wrap-ignore


cdef extern from "<OpenMS/FORMAT/DATAACCESS/SwathFileConsumer.h>" namespace "OpenMS":

    cdef cppclass CachedSwathFileConsumer(FullSwathFileConsumer):
        # wrap-inherits:
        #    FullSwathFileConsumer

        CachedSwathFileConsumer() nogil except + #wrap-ignore
        CachedSwathFileConsumer(CachedSwathFileConsumer &) nogil except + # wrap-ignore
        CachedSwathFileConsumer(String cachedir, String basename, 
                                Size nr_ms1_spectra, libcpp_vector[int] nr_ms2_spectra)

cdef extern from "<OpenMS/FORMAT/DATAACCESS/SwathFileConsumer.h>" namespace "OpenMS":
    
    cdef cppclass MzMLSwathFileConsumer(FullSwathFileConsumer) :
        # wrap-inherits:
        #  FullSwathFileConsumer

        MzMLSwathFileConsumer() nogil except + # wrap-ignore
        MzMLSwathFileConsumer(MzMLSwathFileConsumer) nogil except + #wrap-ignore
        MzMLSwathFileConsumer(String cachedir,
                              String basename, 
                              Size nr_ms1_spectra, 
                              libcpp_vector[ int ] nr_ms2_spectra) nogil except +
        MzMLSwathFileConsumer(libcpp_vector[ SwathMap ] known_window_boundaries, 
                              String cachedir,
                              String basename,
                              Size nr_ms1_spectra, 
                              libcpp_vector[ int ] nr_ms2_spectra) nogil except +





import pyopenms
import unittest

class TestLightTargetedExperiment(unittest.TestCase):


    def setUp(self):

        lt = pyopenms.LightTransition()
        lt.fragment_charge = 2
        lt.transition_name = b"X"
        lt.peptide_ref = b"Y"
        lt.library_intensity = 12.0
        lt.product_mz = 22.0
        self.lt = lt

        lm = pyopenms.LightModification()
        lm.location = 13
        lm.unimod_id = 4

        self.lm = lm

        lpep = pyopenms.LightCompound()
        lpep.rt = 12.0
        lpep.charge =  2
        lpep.sequence =  b"SEQ"
        lpep.protein_refs =  [b"REF"]


        lpep.modifications = [lm]
        self.lpep = lpep

        lprot = pyopenms.LightProtein()
        lprot.id = b"1234"
        lprot.sequence = b"ABC"

        self.lprot = lprot

        lte = pyopenms.LightTargetedExperiment()
        lte.compounds = [self.lpep]
        lte.proteins = [self.lprot]
        lte.transitions = [self.lt]

        self.lte = lte


    @staticmethod
    def _test_light_transition(lt):
        assert lt.fragment_charge == 2
        assert lt.getProductChargeState() == lt.fragment_charge
        assert lt.transition_name == b"X"
        assert lt.peptide_ref == b"Y"
        assert lt.library_intensity == 12.0
        assert lt.product_mz == 22.0

    @staticmethod
    def _test_light_modification(lm):
        assert lm.location == 13
        assert lm.unimod_id == 4

    @staticmethod
    def _test_light_peptide(lpep):

        assert lpep.rt == 12.0
        assert lpep.charge ==  2
        assert lpep.sequence ==  b"SEQ"
        assert lpep.protein_refs ==  [b"REF"]

        mod, = lpep.modifications
        TestLightTargetedExperiment._test_light_modification(mod)

    @staticmethod
    def _test_light_protein(lprot):
        assert lprot.id == b"1234"
        assert lprot.sequence == b"ABC"

    def test_light_transition(self):
        TestLightTargetedExperiment._test_light_transition(self.lt)


    def test_light_modification(self):
        TestLightTargetedExperiment._test_light_modification(self.lm)


    def test_light_peptide(self):
        TestLightTargetedExperiment._test_light_peptide(self.lpep)


    def test_light_protein(self):
        TestLightTargetedExperiment._test_light_protein(self.lprot)

    def test_light_targeted_experiment(self):
        lprot, = self.lte.proteins
        lpep, = self.lte.compounds
        ltrans, = self.lte.transitions

        TestLightTargetedExperiment._test_light_protein(lprot)
        TestLightTargetedExperiment._test_light_peptide(lpep)
        TestLightTargetedExperiment._test_light_transition(ltrans)

        ltrans, = self.lte.getTransitions()
        TestLightTargetedExperiment._test_light_transition(ltrans)




from libcpp.vector cimport vector as libcpp_vector
from ConsensusMap cimport *

cdef extern from "<OpenMS/ANALYSIS/MAPMATCHING/ConsensusMapNormalizerAlgorithmThreshold.h>" namespace "OpenMS":

    cdef cppclass ConsensusMapNormalizerAlgorithmThreshold:

        ConsensusMapNormalizerAlgorithmThreshold() nogil except +
        ConsensusMapNormalizerAlgorithmThreshold(ConsensusMapNormalizerAlgorithmThreshold) nogil except + #wrap-ignore

        libcpp_vector[double] computeCorrelation(ConsensusMap & input_map,
                                                 double ratio_threshold,
                                                 const String & acc_filter,
                                                 const String & desc_filter) nogil except +

        void normalizeMaps(ConsensusMap & input_map,
                           libcpp_vector[double] & ratios) nogil except +




from Types cimport *

cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLDataStructs.h>" namespace "OpenMS":

    cdef cppclass OPXLDataStructs "OpenMS::OPXLDataStructs":
        OPXLDataStructs(OPXLDataStructs) nogil except + # wrap-ignore
        OPXLDataStructs() nogil except + # wrap-ignore

cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLDataStructs.h>" namespace "OpenMS::OPXLDataStructs":
    cdef enum ProteinProteinCrossLinkType "OpenMS::OPXLDataStructs::ProteinProteinCrossLinkType":
        #wrap-attach:
        #    OPXLDataStructs
        CROSS
        MONO
        LOOP
        NUMBER_OF_CROSS_LINK_TYPES

cdef extern from "<OpenMS/ANALYSIS/XLMS/OPXLDataStructs.h>" namespace "OpenMS::OPXLDataStructs":
    cdef enum PeptidePosition "OpenMS::OPXLDataStructs::PeptidePosition":
        #wrap-attach:
        #    OPXLDataStructs
        INTERNAL
        C_TERM
        N_TERM



from Types cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from Types cimport *
from FileHandler cimport *
from DataValue cimport *
from PeptideIdentification cimport *
from ProteinIdentification cimport *

cdef extern from "<OpenMS/ANALYSIS/ID/PercolatorFeatureSetHelper.h>" namespace "OpenMS":
    
    cdef cppclass PercolatorFeatureSetHelper "OpenMS::PercolatorFeatureSetHelper":

        PercolatorFeatureSetHelper(PercolatorFeatureSetHelper) nogil except + #wrap-ignore

        void concatMULTISEPeptideIds(libcpp_vector[ PeptideIdentification ] & all_peptide_ids, libcpp_vector[ PeptideIdentification ] & new_peptide_ids, String search_engine) nogil except +
        void mergeMULTISEPeptideIds(libcpp_vector[ PeptideIdentification ] & all_peptide_ids, libcpp_vector[ PeptideIdentification ] & new_peptide_ids, String search_engine) nogil except +
        void mergeMULTISEProteinIds(libcpp_vector[ ProteinIdentification ] & all_protein_ids, libcpp_vector[ ProteinIdentification ] & new_protein_ids) nogil except +
        void addMSGFFeatures(libcpp_vector[ PeptideIdentification ] & peptide_ids, StringList & feature_set) nogil except +
        void addXTANDEMFeatures(libcpp_vector[ PeptideIdentification ] & peptide_ids, StringList & feature_set) nogil except +
        void addCOMETFeatures(libcpp_vector[ PeptideIdentification ] & peptide_ids, StringList & feature_set) nogil except +
        void addMASCOTFeatures(libcpp_vector[ PeptideIdentification ] & peptide_ids, StringList & feature_set) nogil except +
        void addMULTISEFeatures(libcpp_vector[ PeptideIdentification ] & peptide_ids, StringList & search_engines_used, StringList & feature_set, bool complete_only, bool limits_imputation) nogil except +
        void addCONCATSEFeatures(libcpp_vector[ PeptideIdentification ] & peptide_id_list, StringList & search_engines_used, StringList & feature_set) nogil except +
        void checkExtraFeatures(libcpp_vector[ PeptideHit ] & psms, StringList & extra_features) nogil except +




from Peak1D cimport *
from Feature cimport *
from FeatureMap cimport *
from MSExperiment cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *
from FeatureFinder cimport *
from ConsensusMap cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/TRANSFORMATIONS/FEATUREFINDER/FeatureFinderMultiplexAlgorithm.h>" namespace "OpenMS":

    cdef cppclass FeatureFinderMultiplexAlgorithm(DefaultParamHandler):

        # wrap-inherits:
        #    DefaultParamHandler
        FeatureFinderMultiplexAlgorithm()      nogil except +

        void run(MSExperiment& exp, bool progress) nogil except +

        FeatureMap getFeatureMap() nogil except +

        ConsensusMap getConsensusMap() nogil except +



from Types cimport *
from MSSpectrum cimport *
from MSExperiment cimport *
from Peak1D cimport *
from PeptideIdentification cimport *
from ProteinIdentification cimport *
from SpectrumLookup cimport *

cdef extern from "<OpenMS/METADATA/SpectrumMetaDataLookup.h>" namespace "OpenMS":

    cdef cppclass SpectrumMetaDataLookup(SpectrumLookup):
        # wrap-inherits:
        #  SpectrumLookup

        SpectrumMetaDataLookup() nogil except +
        # SpectrumMetaDataLookup(SpectrumMetaDataLookup) nogil except + # private

        void readSpectra(MSExperiment spectra, String scan_regexp, bool get_precursor_rt) nogil except +

        void getSpectrumMetaData(Size index, SpectrumMetaData& meta) nogil except +

        void getSpectrumMetaData(String spectrum_ref, SpectrumMetaData& meta) nogil except +

        void getSpectrumMetaData(String spectrum_ref, SpectrumMetaData& meta, unsigned char flags) nogil except +

        void setSpectraDataRef(const String & spectra_data) nogil except +

#
## wrap static methods
#
cdef extern from "<OpenMS/METADATA/SpectrumMetaDataLookup.h>" namespace "OpenMS::SpectrumMetaDataLookup":

    ###   TODO: Missing optional parameters:
    ###   boost::regex& scan_regexp 
    ###   std::map<Size, double>& precursor_rts 
    void getSpectrumMetaData(MSSpectrum spectrum,
                             SpectrumMetaData& meta) nogil except + # wrap-attach:SpectrumMetaDataLookup

    bool addMissingRTsToPeptideIDs(libcpp_vector[PeptideIdentification], 
                                   String filename, bool stop_on_error) nogil except + # wrap-attach:SpectrumMetaDataLookup

    bool addMissingSpectrumReferences(libcpp_vector[PeptideIdentification], 
                                   String filename, bool stop_on_error, 
                                   bool override_spectra_data, 
                                   bool override_spectra_references,
                                   libcpp_vector[ProteinIdentification] proteins) nogil except + # wrap-attach:SpectrumMetaDataLookup

cdef extern from "<OpenMS/METADATA/SpectrumMetaDataLookup.h>" namespace "OpenMS::SpectrumMetaDataLookup":

    cdef cppclass SpectrumMetaData "OpenMS::SpectrumMetaDataLookup::SpectrumMetaData":

      SpectrumMetaData() nogil except +
      SpectrumMetaData(SpectrumMetaData) nogil except + # wrap-ignore

      double rt
      double precursor_rt
      double precursor_mz
      Int precursor_charge
      Size ms_level
      Int scan_number
      String native_id




from Types cimport *
from smart_ptr cimport shared_ptr
from OpenSwathDataStructures cimport *
from ISpectrumAccess cimport *

cdef extern from "<OpenMS/OPENSWATHALGO/DATAACCESS/SwathMap.h>" namespace "OpenSwath":

    cdef cppclass SwathMap:

        SwathMap() nogil except +
        SwathMap(SwathMap) nogil except +

        double lower
        double upper
        double center
        bool ms1

        # COMMENT: access through
        # COMMENT:  - getSpectrumPtr
        # COMMENT:  - setSpectrumPtr
        shared_ptr[ISpectrumAccess] sptr # wrap-ignore




from Types cimport *
from ProgressLogger cimport *
from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from AASequence cimport *
from ResidueModification cimport *
from StringList cimport *

cdef extern from "<OpenMS/ANALYSIS/RNPXL/ModifiedPeptideGenerator.h>" namespace "OpenMS":

    cdef cppclass ModifiedPeptideGenerator:

        ModifiedPeptideGenerator() nogil except +
        ModifiedPeptideGenerator(ModifiedPeptideGenerator) nogil except + 

    cdef cppclass ModifiedPeptideGenerator_MapToResidueType "OpenMS::ModifiedPeptideGenerator::MapToResidueType":

        ModifiedPeptideGenerator_MapToResidueType() nogil except +
        ModifiedPeptideGenerator_MapToResidueType(ModifiedPeptideGenerator_MapToResidueType) nogil except + #wrap-ignore


    ## wrap static methods
    cdef extern from "<OpenMS/ANALYSIS/RNPXL/ModifiedPeptideGenerator.h>" namespace "OpenMS::ModifiedPeptideGenerator":
        ModifiedPeptideGenerator_MapToResidueType getModifications(const StringList& modNames) nogil except +

        void applyFixedModifications(const ModifiedPeptideGenerator_MapToResidueType& fixed_mods, AASequence& peptide) nogil except +

        void applyVariableModifications(const ModifiedPeptideGenerator_MapToResidueType& var_mods, 
          const AASequence& peptide, 
          Size max_variable_mods_per_peptide, 
          libcpp_vector[AASequence]& all_modified_peptides, 
          bool keep_original) nogil except +




import unittest
import os

import pyopenms

class TestOpenSwathDataStructures(unittest.TestCase):

    def setUp(self):
        pass

    def test_spectrum(self):
        spectrum = pyopenms.Interfaces.Spectrum()
        mz_exp = [1,2,3]
        int_exp = [4,5,6]

        spectrum.setMZArray(mz_exp)
        spectrum.setIntensityArray(int_exp)
        mz = spectrum.getMZArray()
        intensity = spectrum.getIntensityArray()

        for m,e in zip(mz, mz_exp):
            self.assertAlmostEqual(m,e)

        for i,e in zip(intensity, int_exp):
            self.assertAlmostEqual(i,e)

    def test_chromatogram(self):
        chromatogram = pyopenms.Interfaces.Chromatogram()
        rt_exp = [1,2,3]
        int_exp = [4,5,6]

        chromatogram.setTimeArray(rt_exp)
        chromatogram.setIntensityArray(int_exp)
        time = chromatogram.getTimeArray()
        intensity = chromatogram.getIntensityArray()

        for m,e in zip(time, rt_exp):
            self.assertAlmostEqual(m,e)

        for i,e in zip(intensity, int_exp):
            self.assertAlmostEqual(i,e)

if __name__ == '__main__':
    unittest.main()



from Types cimport *
from CsiAdapterHit cimport *
from String cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/CsiFingerIdMzTabWriter.h>" namespace "OpenMS::CsiFingerIdMzTabWriter":
    
    cdef cppclass CsiAdapterIdentification "OpenMS::CsiFingerIdMzTabWriter::CsiAdapterIdentification":
        CsiAdapterIdentification() nogil except +
        CsiAdapterIdentification(CsiAdapterIdentification) nogil except + #wrap-ignore
        int scan_index
        int scan_number
        String feature_id
        libcpp_vector[ CsiAdapterHit ] hits




from __future__ import print_function
from autowrap.Code import Code
from autowrap.ConversionProvider import (TypeConverterBase,
                                         mangle,
                                         StdMapConverter)

class OpenMSDPosition2(TypeConverterBase):

    def get_base_types(self):
        return "DPosition2",

    def matches(self, cpp_type):
        return  not cpp_type.is_ptr

    def matching_python_type(self, cpp_type):
        return ""

    def type_check_expression(self, cpp_type, argument_var):
        return "len(%s) == 2 and isinstance(%s[0], (int, float)) "\
                "and isinstance(%s[1], (int, float))"\
                % (argument_var, argument_var, argument_var)

    def input_conversion(self, cpp_type, argument_var, arg_num):
        dp = "_dp_%s" % arg_num
        code = Code().add("""
            |cdef _DPosition2 $dp
            |$dp[0] = <float>$argument_var[0]
            |$dp[1] = <float>$argument_var[1]
        """, locals())
        cleanup = ""
        if cpp_type.is_ref:
            cleanup = Code().add("""
            |$cpp_type[0] = $dp[0]
            |$cpp_type[1] = $dp[1]
            """, locals())
        call_as = dp
        return code, call_as, cleanup

    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):
        # this one is slow as it uses construction of python type DataValue for
        # delegating conversion to this type, which reduces code below:
        return Code().add("""
                    |$output_py_var = [$input_cpp_var[0], $input_cpp_var[1]]
                """, locals())


class OpenMSDPosition2Vector(TypeConverterBase):

    def get_base_types(self):
        return "libcpp_vector",

    def matches(self, cpp_type):
        inner_t, = cpp_type.template_args
        return inner_t == "DPosition2"

    def matching_python_type(self, cpp_type):
        return "np.ndarray[np.float32_t,ndim=2]"

    def type_check_expression(self, cpp_type, argument_var):
        return "%s.shape[1] == 2" % argument_var

    def input_conversion(self, cpp_type, argument_var, arg_num):
        dp = "_dp_%s" % arg_num
        vec ="_dp_vec_%s" % arg_num
        ii = "_dp_ii_%s" % arg_num
        N = "_dp_N_%s" % arg_num
        code = Code().add("""
            |cdef libcpp_vector[_DPosition2] $vec
            |cdef _DPosition2 $dp
            |cdef int $ii
            |cdef int $N = $argument_var.shape[0]
            |for $ii in range($N):
            |    $dp[0] = $argument_var[$ii,0]
            |    $dp[1] = $argument_var[$ii,1]
            |    $vec.push_back($dp)
        """, locals())
        cleanup = ""
        if cpp_type.is_ref:
            it = "_dp_it_%s" % arg_num
            n = "_dp_n_%s" % arg_num
            cleanup = Code().add("""
            |$n = $vec.size()
            |$argument_var.resize(($n,2))

            |$n = $vec.size()
            |cdef libcpp_vector[_DPosition2].iterator $it = $vec.begin()
            |$ii = 0
            |while $it != $vec.end():
            |     $argument_var[$ii, 0] = deref($it)[0]
            |     $argument_var[$ii, 1] = deref($it)[1]
            |     inc($it)
            |     $ii += 1

            """, locals())
        call_as = vec
        return code, call_as, cleanup

    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):
        # this one is slow as it uses construction of python type DataValue for
        # delegating conversion to this type, which reduces code below:
        it = "_out_it_dpos_vec"
        n  = "_out_n_dpos_vec"
        ii = "_out_ii_dpos_vec"
        return Code().add("""
         |cdef int $n = $input_cpp_var.size()
         |cdef $output_py_var = np.zeros([$n,2], dtype=np.float32)
         |cdef libcpp_vector[_DPosition2].iterator $it = $input_cpp_var.begin()
         |cdef int $ii = 0
         |while $it != $input_cpp_var.end():
         |     $output_py_var[$ii, 0] = deref($it)[0]
         |     $output_py_var[$ii, 1] = deref($it)[1]
         |     inc($it)
         |     $ii += 1
         """, locals())


class OpenMSDataValue(TypeConverterBase):

    def get_base_types(self):
        return "DataValue",

    def matches(self, cpp_type):
        return  not cpp_type.is_ptr and not cpp_type.is_ref

    def matching_python_type(self, cpp_type):
        return ""

    def type_check_expression(self, cpp_type, argument_var):
        return "isinstance(%s, (int, long, float, list, bytes, str, unicode))" % argument_var

    def input_conversion(self, cpp_type, argument_var, arg_num):
        call_as = "deref(DataValue(%s).inst.get())" % argument_var
        return "", call_as, ""

    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):
        # this one is slow as it uses construction of python type DataValue for
        # delegating conversion to this type, which reduces code below:
        return Code().add("""
                    |cdef DataValue _value = DataValue.__new__(DataValue)
                    |_value.inst = shared_ptr[_DataValue](new _DataValue($input_cpp_var))
                    |cdef int _type = $input_cpp_var.valueType()
                    |cdef object $output_py_var
                    |if _type == DataType.STRING_VALUE:
                    |    $output_py_var = _value.toString()
                    |elif _type == DataType.INT_VALUE:
                    |    $output_py_var = _value.toInt()
                    |elif _type == DataType.DOUBLE_VALUE:
                    |    $output_py_var = _value.toDouble()
                    |elif _type == DataType.INT_LIST:
                    |    $output_py_var = _value.toIntList()
                    |elif _type == DataType.DOUBLE_LIST:
                    |    $output_py_var = _value.toDoubleList()
                    |elif _type == DataType.STRING_LIST:
                    |    $output_py_var = _value.toStringList()
                    |elif _type == DataType.EMPTY_VALUE:
                    |    $output_py_var = None
                    |else:
                    |    raise Exception("DataValue instance has invalid value type %d" % _type)
                """, locals())


class OpenMSStringConverter(TypeConverterBase):

    def get_base_types(self):
        return "String",

    def matches(self, cpp_type):
        return  not cpp_type.is_ptr

    def matching_python_type(self, cpp_type):
        # We allow bytes, unicode, str and String
        return ""

    def type_check_expression(self, cpp_type, argument_var):
        # Need to treat ptr and reference differently as these may be modified
        # and the results needs to be available in Python
        if (cpp_type.is_ptr or cpp_type.is_ref) and not cpp_type.is_const:
            return "isinstance(%s, String)" % (argument_var)

        # Allow conversion from unicode str, bytes and OpenMS::String
        return "(isinstance(%s, str) or isinstance(%s, unicode) or isinstance(%s, bytes) or isinstance(%s, String))" % (
            argument_var,argument_var,argument_var, argument_var)

    def input_conversion(self, cpp_type, argument_var, arg_num):

        # See ./src/pyOpenMS/addons/ADD_TO_FIRST.pyx for declaration of convString
        call_as = "deref((convString(%s)).get())" % argument_var
        cleanup = ""
        code = ""
        # Need to treat ptr and reference differently as these may be modified
        # and the results needs to be available in Python
        if cpp_type.is_ptr and not cpp_type.is_const:
            call_as = "((<String>%s).inst.get())" % argument_var
        if cpp_type.is_ref and not cpp_type.is_const:
            call_as = "deref((<String>%s).inst.get())" % argument_var
        return code, call_as, cleanup

    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):

        # See ./src/pyOpenMS/addons/ADD_TO_FIRST.pyx for declaration of convOutputString
        return "%s = convOutputString(%s)" % (output_py_var, input_cpp_var)

class AbstractOpenMSListConverter(TypeConverterBase):

    def __init__(self):
        # mark as abstract:
        raise NotImplementedError()

    def get_base_types(self):
        return self.openms_type,

    def matches(self, cpp_type):
        return not cpp_type.is_ptr

    def matching_python_type(self, cpp_type):
        return "list"

    def type_check_expression(self, cpp_type, argument_var):
        return\
            "isinstance(%s, list) and all(isinstance(li, %s) for li in %s)"\
            % (argument_var, self.inner_py_type, argument_var)

    def input_conversion(self, cpp_type, argument_var, arg_num):
        temp_var = "v%d" % arg_num
        t = self.inner_cpp_type
        ltype = self.openms_type
        code = Code().add("""
                |cdef libcpp_vector[$t] _$temp_var = $argument_var
                |cdef _$ltype $temp_var = _$ltype(_$temp_var)
                """, locals())
        cleanup = ""
        if cpp_type.is_ref:
            cleanup_code = Code().add("""
                    |replace = []
                    |cdef int i, n
                    |n = $temp_var.size()
                    |for i in range(n):
                    |    replace.append(<$t>$temp_var.at(i))
                    |$argument_var[:] = replace
                    """, locals())
        # here we inject special behavoir for testing if this converter
        # was called !
        call_as = "(%s)" % temp_var
        return code, call_as, cleanup

    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):
        t = self.inner_cpp_type
        code = Code().add("""
            |$output_py_var = []
            |cdef int i, n
            |n = $input_cpp_var.size()
            |for i in range(n):
            |    $output_py_var.append(<$t>$input_cpp_var.at(i))
            """, locals())
        return code

class OpenMSIntListConverter(AbstractOpenMSListConverter):

    openms_type = "IntList"
    inner_py_type = "int"
    inner_cpp_type = "int"
    # mark as non abstract:
    def __init__(self):
        pass

class OpenMSDoubleListConverter(AbstractOpenMSListConverter):

    openms_type = "DoubleList"
    inner_py_type = "float"
    inner_cpp_type = "double"
    # mark as non abstract:
    def __init__(self):
        pass

class StdVectorStringConverter(TypeConverterBase):

    def get_base_types(self):
        return "libcpp_vector",

    def matches(self, cpp_type):
        inner_t, = cpp_type.template_args
        return inner_t == "String"

    def matching_python_type(self, cpp_type):
        return "list"

    def type_check_expression(self, cpp_type, arg_var):
        return Code().add("""
          |isinstance($arg_var, list) and all(isinstance(i, bytes) for i in
          + $arg_var)
          """, locals()).render()

    def input_conversion(self, cpp_type, argument_var, arg_num):
        temp_var = "v%d" % arg_num
        temp_it = "it_%d" % arg_num
        item = "item%d" % arg_num
        code = Code().add("""
            |cdef libcpp_vector[_String] * $temp_var = new libcpp_vector[_String]()
            |cdef bytes $item
            |for $item in $argument_var:
            |   $temp_var.push_back(_String(<char *>$item))
            """, locals())
        if cpp_type.is_ref:
            cleanup_code = Code().add("""
                |replace = []
                |cdef libcpp_vector[_String].iterator $temp_it = $temp_var.begin()
                |while $temp_it != $temp_var.end():
                |   replace.append(<char*>deref($temp_it).c_str())
                |   inc($temp_it)
                |$argument_var[:] = replace
                |del $temp_var
                """, locals())
        else:
            cleanup_code = "del %s" % temp_var
        return code, "deref(%s)" % temp_var, cleanup_code

    def call_method(self, res_type, cy_call_str):
        return "_r = %s" % (cy_call_str)

    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):

        if cpp_type.is_ptr:
            raise AssertionError()

        it = mangle("it_" + input_cpp_var)
        item = mangle("item_" + output_py_var)
        code = Code().add("""
            |$output_py_var = []
            |cdef libcpp_vector[_String].iterator $it = $input_cpp_var.begin()
            |while $it != $input_cpp_var.end():
            |   $output_py_var.append(<char*>deref($it).c_str())
            |   inc($it)
            """, locals())
        return code

class OpenMSStringListConverter(StdVectorStringConverter):
    """
    StringList is now a std::vector<String> so we can directly
    inherit all methods from StdVectorStringConverter but need 
    to add the matching rules for StringList.
    """
    openms_type = "StringList"
    inner_py_type = "bytes"
    inner_cpp_type = "libcpp_string"
    # mark as non abstract:
    def __init__(self):
        pass

    def get_base_types(self):
        return self.openms_type,

    def matches(self, cpp_type):
        return not cpp_type.is_ptr

    def matching_python_type(self, cpp_type):
        return "list"

    def type_check_expression(self, cpp_type, argument_var):
        return\
            "isinstance(%s, list) and all(isinstance(li, %s) for li in %s)"\
            % (argument_var, self.inner_py_type, argument_var)

class StdSetStringConverter(TypeConverterBase):

    def get_base_types(self):
        return "libcpp_set",

    def matches(self, cpp_type):
        inner_t, = cpp_type.template_args
        return inner_t == "String"

    def matching_python_type(self, cpp_type):
        return "set"

    def type_check_expression(self, cpp_type, arg_var):
        return Code().add("""
          |isinstance($arg_var, set) and all(isinstance(i, bytes) for i in
          + $arg_var)
          """, locals()).render()

    def input_conversion(self, cpp_type, argument_var, arg_num):
        temp_var = "v%d" % arg_num
        item = "item%d" % arg_num
        code = Code().add("""
            |cdef libcpp_set[_String] * $temp_var = new libcpp_set[_String]()
            |cdef bytes $item
            |for $item in $argument_var:
            |   $temp_var.insert(_String(<char *>$item))
            """, locals())
        if cpp_type.is_ref:
            cleanup_code = Code().add("""
                |cdef replace = set()
                |cdef libcpp_set[_String].iterator it = $temp_var.begin()
                |while it != $temp_var.end():
                |   replace.add(<char*>deref(it).c_str())
                |   inc(it)
                |$argument_var.clear()
                |$argument_var.update(replace)
                |del $temp_var
                """, locals())
        else:
            cleanup_code = "del %s" % temp_var
        return code, "deref(%s)" % temp_var, cleanup_code

    def call_method(self, res_type, cy_call_str):
        return "_r = %s" % (cy_call_str)


    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):

        if cpp_type.is_ptr:
            raise AssertionError()

        it = mangle("it_" + input_cpp_var)
        item = mangle("item_" + output_py_var)
        code = Code().add("""
            |$output_py_var = set()
            |cdef libcpp_set[_String].iterator $it = $input_cpp_var.begin()
            |while $it != $input_cpp_var.end():
            |   $output_py_var.add(<char*>deref($it).c_str())
            |   inc($it)
            """, locals())
        return code


class OpenMSMapConverter(StdMapConverter):

    def get_base_types(self):
        return "Map",

    def matches(self, cpp_type):
        return True

    def matching_python_type(self, cpp_type):
        return "dict"

    def type_check_expression(self, cpp_type, arg_var):
        tt_key, tt_value = cpp_type.template_args
        inner_conv_1 = self.converters.get(tt_key)
        inner_conv_2 = self.converters.get(tt_value)

        if inner_conv_1 is None:
            raise Exception("arg type %s not supported" % tt_key)
        if inner_conv_2 is None:
            raise Exception("arg type %s not supported" % tt_value)

        inner_check_1 = inner_conv_1.type_check_expression(tt_key, "k")
        inner_check_2 = inner_conv_2.type_check_expression(tt_value, "v")

        return Code().add("""
          |isinstance($arg_var, dict)
          + and all($inner_check_1 for k in $arg_var.keys())
          + and all($inner_check_2 for v in $arg_var.values())
          """, locals()).render()

    def input_conversion(self, cpp_type, argument_var, arg_num):
        tt_key, tt_value = cpp_type.template_args
        temp_var = "v%d" % arg_num

        cy_tt_key = self.converters.cython_type(tt_key)
        cy_tt_value = self.converters.cython_type(tt_value)

        if cy_tt_key.is_enum:
            key_conv = "<%s> key" % cy_tt_key
        elif tt_key.base_type in self.converters.names_to_wrap:
            raise Exception("can not handle wrapped classes as keys in map")
        else:
            key_conv = "<%s> key" % cy_tt_key

        if cy_tt_value.is_enum:
            value_conv = "<%s> value" % cy_tt_value
        elif tt_value.base_type in self.converters.names_to_wrap:
            value_conv = "deref((<%s>value).inst.get())" % tt_value.base_type
        else:
            value_conv = "<%s> value" % cy_tt_value

        code = Code().add("""
            |cdef _Map[$cy_tt_key, $cy_tt_value] * $temp_var = new
            + _Map[$cy_tt_key, $cy_tt_value]()

            |for key, value in $argument_var.items():
            |   deref($temp_var)[$key_conv] = $value_conv
            """, locals())

        if cpp_type.is_ref:
            replace = mangle("replace_" + argument_var)
            it = mangle("it_" + argument_var)

            if cy_tt_key.is_enum:
                key_conv = "<%s> deref(%s).first" % (cy_tt_key, it)
            elif tt_key.base_type in self.converters.names_to_wrap:
                raise Exception("can not handle wrapped classes as keys in map")
            else:
                key_conv = "<%s> deref(%s).first" % (cy_tt_key, it)

            if not cy_tt_value.is_enum and tt_value.base_type in self.converters.names_to_wrap:
                cy_tt = tt_value.base_type
                item = mangle("item_" + argument_var)
                cleanup_code = Code().add("""
                    |cdef $replace = dict()
                    |cdef _Map[$cy_tt_key, $cy_tt_value].iterator $it = $temp_var.begin()
                    |cdef $cy_tt $item
                    |while $it != $temp_var.end():
                    |   $item = $cy_tt.__new__($cy_tt)
                    |   $item.inst = shared_ptr[$cy_tt_value](new $cy_tt_value((deref($it)).second))
                    |   $replace[$key_conv] = $item
                    |   inc($it)
                    |$argument_var.clear()
                    |$argument_var.update($replace)
                    |del $temp_var
                    """, locals())
            else:
                value_conv = "<%s> deref(%s).second" % (cy_tt_value, it)
                cleanup_code = Code().add("""
                    |cdef $replace = dict()
                    |cdef _Map[$cy_tt_key, $cy_tt_value].iterator $it = $temp_var.begin()
                    |while $it != $temp_var.end():
                    |   $replace[$key_conv] = $value_conv
                    |   inc($it)
                    |$argument_var.clear()
                    |$argument_var.update($replace)
                    |del $temp_var
                    """, locals())
        else:
            cleanup_code = "del %s" % temp_var

        return code, "deref(%s)" % temp_var, cleanup_code

    def call_method(self, res_type, cy_call_str):
        return "_r = %s" % (cy_call_str)

    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):

        if cpp_type.is_ptr:
            raise AssertionError()

        tt_key, tt_value = cpp_type.template_args
        cy_tt_key = self.converters.cython_type(tt_key)
        cy_tt_value = self.converters.cython_type(tt_value)

        it = mangle("it_" + input_cpp_var)

        if not cy_tt_key.is_enum and tt_key.base_type in self.converters.names_to_wrap:
            raise Exception("can not handle wrapped classes as keys in map")
        else:
            key_conv = "<%s>(deref(%s).first)" % (cy_tt_key, it)

        if not cy_tt_value.is_enum and tt_value.base_type in self.converters.names_to_wrap:
            cy_tt = tt_value.base_type
            item = mangle("item_" + output_py_var)
            code = Code().add("""
                |$output_py_var = dict()
                |cdef _Map[$cy_tt_key, $cy_tt_value].iterator $it = $input_cpp_var.begin()
                |cdef $cy_tt $item
                |while $it != $input_cpp_var.end():
                |   $item = $cy_tt.__new__($cy_tt)
                |   $item.inst = shared_ptr[$cy_tt_value](new $cy_tt_value((deref($it)).second))
                |   $output_py_var[$key_conv] = $item
                |   inc($it)
                """, locals())
            return code
        else:
            value_conv = "<%s>(deref(%s).second)" % (cy_tt_value, it)
            code = Code().add("""
                |$output_py_var = dict()
                |cdef _Map[$cy_tt_key, $cy_tt_value].iterator $it = $input_cpp_var.begin()
                |while $it != $input_cpp_var.end():
                |   $output_py_var[$key_conv] = $value_conv
                |   inc($it)
                """, locals())
            return code


import time

class CVTermMapConverter(TypeConverterBase):

    def get_base_types(self):
        return "Map",

    def matches(self, cpp_type):
        # print(str(cpp_type), "Map[String,libcpp_vector[CVTerm]]")
        return str(cpp_type) == "Map[String,libcpp_vector[CVTerm]]" \
           or  str(cpp_type) == "Map[String,libcpp_vector[CVTerm]] &"

    def matching_python_type(self, cpp_type):
        return "dict"

    def type_check_expression(self, cpp_type, arg_var):
        return Code().add("""
          |isinstance($arg_var, dict)
          + and all(isinstance(k, bytes) for k in $arg_var.keys())
          + and all(isinstance(v, list) for v in $arg_var.values())
          + and all(isinstance(vi, CVTerm) for v in $arg_var.values() for vi in
          v)
          """, locals()).render()

    def input_conversion(self, cpp_type, argument_var, arg_num):

        map_name = "_map_%d" % arg_num
        v_vec = "_v_vec_%d" % arg_num
        v_ptr = "_v_ptr_%d" % arg_num
        v_i = "_v_i_%d" % arg_num
        k_string = "_k_str_%d" % arg_num

        code = Code().add("""
                |cdef Map[_String, libcpp_vector[_CVTerm]] $map_name
                |cdef libcpp_vector[_CVTerm] $v_vec
                |cdef _String $k_string
                |cdef CVTerm $v_i
                |for k, v in $argument_var.items():
                |    $v_vec.clear()
                |    for $v_i in v:
                |        $v_vec.push_back(deref($v_i.inst.get()))
                |    $map_name[_String(<char *>k)] = $v_vec
                """, locals())

        if cpp_type.is_ref:
            replace = "_replace_%d" % arg_num
            outer_it = "outer_it_%d" % arg_num
            inner_it = "inner_it_%d" % arg_num
            item     = "item_%d" % arg_num
            inner_key = "inner_key_%d" % arg_num
            inner_values = "inner_values_%d" % arg_num
            cleanup_code = Code().add("""
                |cdef $replace = dict()
                |cdef Map[_String, libcpp_vector[_CVTerm]].iterator $outer_it
                + = $map_name.begin()

                |cdef libcpp_vector[_CVTerm].iterator $inner_it
                |cdef CVTerm $item
                |cdef bytes $inner_key
                |cdef list $inner_values

                |while $outer_it != $map_name.end():
                |   $inner_key = deref($outer_it).first.c_str()
                |   $inner_values = []
                |   $inner_it = deref($outer_it).second.begin()
                |   while $inner_it != deref($outer_it).second.end():
                |       $item = CVTerm.__new__(CVTerm)
                |       $item.inst = shared_ptr[_CVTerm](new _CVTerm(deref($inner_it)))
                |       $inner_values.append($item)
                |       inc($inner_it)
                |   $replace[$inner_key] = $inner_values
                |   inc($outer_it)

                |$argument_var.clear()
                |$argument_var.update($replace)
                """, locals())
        else:
            cleanup_code = ""
        return code, map_name, cleanup_code


    def call_method(self, res_type, cy_call_str):
        return "_r = %s" % (cy_call_str)

    def output_conversion(self, cpp_type, input_cpp_var, output_py_var):

        rnd = str(id(self))+str(time.time()).split(".")[0]
        outer_it = "outer_it_%s" % rnd
        inner_it = "inner_it_%s" % rnd
        item     = "item_%s" % rnd
        inner_key = "inner_key_%s" % rnd
        inner_values = "inner_values_%s" % rnd

        code = Code().add("""
            |$output_py_var = dict()
            |cdef Map[_String, libcpp_vector[_CVTerm]].iterator $outer_it
            + = $input_cpp_var.begin()
            |cdef libcpp_vector[_CVTerm].iterator $inner_it
            |cdef CVTerm $item
            |cdef bytes $inner_key
            |cdef list $inner_values
            |while $outer_it != $input_cpp_var.end():
            |   $inner_key = deref($outer_it).first.c_str()
            |   $inner_values = []
            |   $inner_it = deref($outer_it).second.begin()
            |   while $inner_it != deref($outer_it).second.end():
            |       $item = CVTerm.__new__(CVTerm)
            |       $item.inst = shared_ptr[_CVTerm](new _CVTerm(deref($inner_it)))
            |       $inner_values.append($item)
            |       inc($inner_it)
            |   $output_py_var[$inner_key] = $inner_values
            |   inc($outer_it)
            """, locals())

        return code





from Types cimport *
from SignalToNoiseEstimator cimport *
from MSSpectrum cimport *
from DefaultParamHandler cimport *
from ProgressLogger cimport *

cdef extern from "<OpenMS/FILTERING/NOISEESTIMATION/SignalToNoiseEstimatorMeanIterative.h>" namespace "OpenMS":
    
    cdef cppclass SignalToNoiseEstimatorMeanIterative[Container]:
        # wrap-instances:
        #   SignalToNoiseEstimatorMeanIterative := SignalToNoiseEstimatorMeanIterative[ MSSpectrum ]

        SignalToNoiseEstimatorMeanIterative() nogil except +
        SignalToNoiseEstimatorMeanIterative(SignalToNoiseEstimatorMeanIterative) nogil except +
        void init(Container & c) nogil except +
        double getSignalToNoise(Size index) nogil except +

cdef extern from "<OpenMS/FILTERING/NOISEESTIMATION/SignalToNoiseEstimatorMeanIterative.h>" namespace "OpenMS::SignalToNoiseEstimatorMeanIterative<MSSpectrum>":
    
    cdef enum IntensityThresholdCalculation "OpenMS::SignalToNoiseEstimatorMeanIterative<MSSpectrum>::IntensityThresholdCalculation":
        # wrap-attach:
        #     SignalToNoiseEstimatorMeanIterative
        MANUAL
        AUTOMAXBYSTDEV
        AUTOMAXBYPERCENT



from Types cimport *
from IsobaricQuantitationMethod cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/ItraqFourPlexQuantitationMethod.h>" namespace "OpenMS":

    cdef cppclass ItraqFourPlexQuantitationMethod(IsobaricQuantitationMethod) :
        # wrap-inherits:
        #  IsobaricQuantitationMethod
        ItraqFourPlexQuantitationMethod() nogil except +
        ItraqFourPlexQuantitationMethod(ItraqFourPlexQuantitationMethod) nogil except +




from Peak1D cimport *
from ChromatogramPeak cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/KERNEL/ChromatogramTools.h>" namespace "OpenMS":

    cdef cppclass ChromatogramTools:
        ChromatogramTools() nogil except +

        void convertChromatogramsToSpectra(
                MSExperiment & epx
                ) nogil except +

        void convertSpectraToChromatograms(
                MSExperiment & epx,
                bool remove_spectra,
                bool force_conversion
                ) nogil except +







    def getMetaValues(self):
        """Cython signature: dict getMetaValues()

        Returns all meta values in a Python dictionary.
        """
        # cdef _DataValue _c_value
        # cdef DataValue _value 
        # cdef int _type

        mmap = {}
        cdef libcpp_vector[_String] keys 
        cdef object py_result

        # get keys for iteration
        self.inst.get().getKeys(keys)

        cdef libcpp_vector[_String].iterator k_it = keys.begin()
        while k_it != keys.end():
            # easy approach: call Python fxn
            py_str = _cast_const_away(<char*>(deref(k_it)).c_str())
            py_result = self.getMetaValue(py_str)
            # # hard approach: do it ourselves
            # _c_value = self.inst.get().getMetaValue(deref(k_it))
            # py_str = _cast_const_away(<char*>(deref(k_it)).c_str())
            # _type = _c_value.valueType()
            # _value = DataValue.__new__(DataValue)
            # _value.inst = shared_ptr[_DataValue](new _DataValue(_c_value))
            # if _type == DataType.STRING_VALUE:
            #     py_result = _value.toString()
            # elif _type == DataType.INT_VALUE:
            #     py_result = _value.toInt()
            # elif _type == DataType.DOUBLE_VALUE:
            #     py_result = _value.toDouble()
            # elif _type == DataType.INT_LIST:
            #     py_result = _value.toIntList()
            # elif _type == DataType.DOUBLE_LIST:
            #     py_result = _value.toDoubleList()
            # elif _type == DataType.STRING_LIST:
            #     py_result = _value.toStringList()
            # elif _type == DataType.EMPTY_VALUE:
            #     py_result = None
            # else:
            #     raise Exception("DataValue instance has invalid value type %d" % _type)

            mmap[ py_str ] = py_result
            inc(k_it)

        return mmap

    def setMetaValues(self, dict mmap):
        """Cython signature: setMetaValues(dict values)

        Sets the meta values given in the Python dictionary.
        """
        self.inst.get().clearMetaInfo() # ensure its empty first
        for k, v in mmap.iteritems():
            self.inst.get().setMetaValue(deref((convString(k)).get()), deref(DataValue(v).inst.get()))
            # self.setMetaValue(k, v)











    def create(self):
        cdef NeutralLossMarker res = NeutralLossMarker.__new__(NeutralLossMarker)
        res.inst = shared_ptr[_NeutralLossMarker]( <_NeutralLossMarker *> self.inst.get().create() )
        return res



from __future__ import print_function

def parse_pxd_file(path):

    import os

    from Cython.Compiler.CmdLine import parse_command_line
    from Cython.Compiler.Main import create_default_resultobj, CompilationSource
    from Cython.Compiler import Pipeline
    from Cython.Compiler.Scanning import FileSourceDescriptor
    from Cython.Compiler.Nodes import CEnumDefNode, CppClassNode, CTypeDefNode, CVarDefNode, CImportStatNode, CDefExternNode
    # from Cython.Compiler.ExprNodes import *


    options, sources = parse_command_line(["--cplus", path])

    path = os.path.abspath(path)
    basename = os.path.basename(path)
    name, ext = os.path.splitext(basename)

    source_desc = FileSourceDescriptor(path, basename)
    source = CompilationSource(source_desc, name, os.getcwd())
    result = create_default_resultobj(source, options)

    context = options.create_context()
    pipeline = Pipeline.create_pyx_pipeline(context, options, result)
    context.setup_errors(options, result)
    root = pipeline[0](source)  # only parser

    def iter_bodies(tree):
        try:
            for n in tree.body.stats[0].stats:
                # cimports at head of file
                yield n
        except Exception:
            pass
        if hasattr(tree.body, "stats"):
            for s in tree.body.stats:
                if isinstance(s, CDefExternNode):
                    body = s.body
                    if hasattr(body, "stats"):
                        for node in body.stats:
                            yield node
                    else:
                        yield body
        elif hasattr(tree.body, "body"):
            body = tree.body.body
            yield body
        else:
            raise Exception("parse_pxd_file failed: no valied .pxd file !")

    lines = open(path).readlines()

    def cimport(b, _, __):
        print ("cimport", b.module_name, "as", b.as_name)

    handlers = { CEnumDefNode : "",
                 CppClassNode : "",
                 CTypeDefNode : "",
                 CVarDefNode  : "",
                 CImportStatNode  : "",
                 }

    # from autowrap.PXDParser import EnumDecl.parseTree, CppClassDecl.parseTree, CTypeDefDecl.parseTree, MethodOrAttributeDecl.parseTree
    # handlers = { CEnumDefNode : EnumDecl.parseTree,
    #              CppClassNode : CppClassDecl.parseTree,
    #              CTypeDefNode : CTypeDefDecl.parseTree,
    #              CVarDefNode  : MethodOrAttributeDecl.parseTree,
    #              CImportStatNode  : cimport,
    #              }

    result = []
    for body in iter_bodies(root):
        handler = handlers.get(type(body))
        if handler is not None:
            # L.info("parsed %s, handler=%s" % (body.__class__, handler.im_self))
            result.append([ body, lines, path ])
        else:
            for node in getattr(body, "stats", []):
                handler = handlers.get(type(node))
                if handler is not None:
                    result.append([ node, lines, path ])
    return result


def create_pxd_file_map(src_path):
    # Searches through all pxd files and matches them to the corresponding
    # OpenMS headers. Note that each header can be mapped to multiple pxd
    # files:
    #
    # pxd_file_matching[ HeaderName ] = set( [PXDFile, PXDFile, ...] )
    #
    import glob, os, re
    pxd_path = os.path.join(src_path, "src/pyOpenMS/pxds/")
    pxds = glob.glob(pxd_path + "/*.pxd")
    pxd_file_matching = {}
    for pfile in pxds:
        filename_rgx = re.compile("cdef extern from \"<([^\"]*)>\"", re.IGNORECASE)
        filematch = [o.group(1) for o in filename_rgx.finditer(open(pfile).read()) ]
        for fm in filematch:
            res = pxd_file_matching.get(fm, set([]))
            res.update([pfile])
            pxd_file_matching[fm] = res

    return pxd_file_matching




from Types cimport *
from libcpp cimport bool
from libcpp.pair cimport pair as libcpp_pair
from libcpp.vector cimport vector as libcpp_vector
from MSSpectrum cimport *
from Types cimport *

cdef extern from "<OpenMS/ANALYSIS/XLMS/XQuestScores.h>" namespace "OpenMS":

    cdef cppclass XQuestScores "OpenMS::XQuestScores":
        XQuestScores(XQuestScores) nogil except + #wrap-ignore
        float preScore(Size matched_alpha, Size ions_alpha, Size matched_beta, Size ions_beta) nogil except +
        float preScore(Size matched_alpha, Size ions_alpha) nogil except +

        double matchOddsScore(MSSpectrum& theoretical_spec,
                              double fragment_mass_tolerance,
                              bool fragment_mass_tolerance_unit_ppm,
                              bool is_xlink_spectrum,
                              Size n_charges) nogil except +


        double logOccupancyProb(MSSpectrum theoretical_spec,
                                Size matched_size,
                                double fragment_mass_tolerance,
                                bool fragment_mass_tolerance_unit_ppm) nogil except +



        double weightedTICScoreXQuest(Size alpha_size, Size beta_size,
                                     double intsum_alpha, double intsum_beta,
                                     double total_current, bool type_is_cross_link) nogil except +

        double weightedTICScore(Size alpha_size, Size beta_size, double intsum_alpha,
                                double intsum_beta, double total_current,
                                bool type_is_cross_link) nogil except +

        double matchedCurrentChain(libcpp_vector[ libcpp_pair[ size_t, size_t ] ] & matched_spec_common,
                                  libcpp_vector[ libcpp_pair[ size_t, size_t ] ] & matched_spec_xlinks,
                                  MSSpectrum & spectrum_common_peaks,
                                  MSSpectrum & spectrum_xlink_peaks) nogil except +

        double totalMatchedCurrent(libcpp_vector[ libcpp_pair[ size_t, size_t ] ] & matched_spec_common_alpha,
                                   libcpp_vector[ libcpp_pair[ size_t, size_t ] ] & matched_spec_common_beta,
                                   libcpp_vector[ libcpp_pair[ size_t, size_t ] ] & matched_spec_xlinks_alpha,
                                   libcpp_vector[ libcpp_pair[ size_t, size_t ] ] & matched_spec_xlinks_beta,
                                   MSSpectrum & spectrum_common_peaks,
                                   MSSpectrum & spectrum_xlink_peaks) nogil except +

        libcpp_vector[ double ] xCorrelation(MSSpectrum & spec1,
                                             MSSpectrum & spec2,
                                             Int maxshift, double tolerance) nogil except +

        double xCorrelationPrescore(MSSpectrum & spec1, MSSpectrum & spec2,double tolerance) nogil except +



from Types cimport *
from String cimport *
from AASequence cimport *

cdef extern from "<OpenMS/CHEMISTRY/EnzymaticDigestionLogModel.h>" namespace "OpenMS":


    cdef cppclass EnzymaticDigestionLogModel:

      EnzymaticDigestionLogModel() nogil except +

      # not wrapped due to name clash with Enzyme.h
      # Enzyme getEnzyme()  nogil except +
      # void setEnzyme(Enzyme enzyme) nogil except +
      # Enzyme getEnzymeByName(String name) nogil except +

      String getEnzymeName() nogil except +
      void setEnzyme(String name) nogil except +

      double getLogThreshold() nogil except +
      void setLogThreshold(double threshold) nogil except +

      void digest(AASequence & protein, libcpp_vector[AASequence] & output) nogil except +
      Size peptideCount(AASequence & protein) nogil except +

# cdef extern from "<OpenMS/CHEMISTRY/EnzymaticDigestionLogModel.h>" namespace "OpenMS::EnzymaticDigestionLogModel":
# 
#     # protected
#     # cdef cppclass BindingSite:
#     #   BindingSite()
#     #   Size position
#     #   String AAname
# 
#     # cdef cppclass CleavageModel:
#     #   CleavageModel()
#     #   double p_cleave
#     #   double p_miss
# 
